/*
 *
 * NExS Connection Library V3.0
 * Copyright 1990, 1991, 1992, 1993, 1994,
 * X Engineering Software Systems Corporation.
 * Copyright 2000, GreyTrout Software, Inc.
 * All rights reserved.
 *
 * $Id: nexs.h,v 1.12 1997/02/01 15:32:40 devb Exp $
 *
 */

#ifndef NEXS_H
#define NEXS_H

#include <X11/Xlib.h>

/******************************************************************************/

/* First, some compiler-specific defines... */

#ifndef NeedFunctionPrototypes
#if defined(FUNCPROTO) || defined(__STDC__) || defined(__cplusplus) || defined(c_plusplus)
#define NeedFunctionPrototypes 1
#else
#define NeedFunctionPrototypes 0
#endif /* __STDC__ */
#endif /* NeedFunctionPrototypes */

#ifdef __cplusplus
extern "C" {    /* let the C++ compiler know that this is a C header file */
#endif


/******************************************************************************/

/* Maximum number of concurrent NExS ports and X11 server connections */
/* NOTE: actual number of concurrent X11 server connections is kernel-limited */

#define	MAX_PORTS	63
#define	MAX_DISPLAYS	63

/******************************************************************************/

/* structures and type definitions used in remote function calls */

#define MIN_ROW		1
#define MAX_ROW		32767
#define MIN_COL		0
#define MAX_COL		4095

#define MAX_TEXT	512	/* max chars in string, including '\0' */
#define MAX_RANGE_TEXT	32	/* max chars in range name, including '\0' */

#define VALID_ROW(R)	((R) >= MIN_ROW && (R) <= MAX_ROW)
#define VALID_COL(C)	((C) >= MIN_COL && (C) <= MAX_COL)

typedef double	Number;

typedef	struct	{
		unsigned short r0, r1;
		unsigned short c0, c1;
		unsigned char  flags;
		} Range;

enum argtypes   {
		ARG_ERROR,
		ARG_CELL,
		ARG_RANGE,
		ARG_NUMBER,
		ARG_STRING,
		ARG_COUNT,
		ARG_UNDEFINED,
		};

typedef	struct	{
		enum argtypes type;
		union	{
			Number number;
			char *string;
			Range range;
			int count;
			} val;
		} Operand;

/* define range flag bits */

#define	CELL_ONLY	1
#define	ABS_C0		2
#define	ABS_R0		4
#define	ABS_C1		8
#define	ABS_R1		16

/******************************************************************************/

/* format codes for nexs_store_formatted_number() */

#define FMT_DEF			'*'
#define FMT_FLOAT		'e'
#define FMT_FIXED		'f'
#define FMT_GEN			'g'
#define FMT_DOLLARS		'$'
#define FMT_COMMA		','
#define FMT_HEX			'h'
#define FMT_LOGIC		'l'
#define FMT_DAY_MONTH_YEAR	'a'
#define FMT_DAY_MONTH		'b'
#define FMT_MONTH_YEAR		'c'
#define FMT_DATE		'd'
#define FMT_HIDDEN		'i'
#define FMT_TIME		't'
#define FMT_PERCENT		'%'
#define FMT_TEXT		'x'

/* font codes for nexs_store_formatted_number() and nexs_set_font() */

#define FNT_DEF			0
#define FNT_NORMAL		1
#define FNT_BOLD		2
#define FNT_ITALIC		3
#define FNT_BOLD_ITALIC		4

/* recalculation flags (nexs_recalc_flags) */

#define	NEXS_RECALC		1
#define NEXS_FORCE_RECALC	2
#define NEXS_FULL_RECALC	4
#define NEXS_MANUAL_MODE	8
#define NEXS_CONSTRAINT_MODE	16
#define NEXS_CONSTRAINT		32
#define NEXS_REMOTE_EVAL	64

/* directions for nexs_scroll_range() */

#define	NEXS_SCROLL_UP		0
#define	NEXS_SCROLL_DOWN	1
#define	NEXS_SCROLL_LEFT	2
#define	NEXS_SCROLL_RIGHT	3

/* return values for nexs_get_value() and nexs_evaluate_expr() */

#define NEXS_EMPTY		0
#define	NEXS_NUMBER		1
#define NEXS_STRING		2
#define NEXS_ERROR		3

/* return values for nexs_get_formula() */

#define NEXS_EMPTY		0
#define	NEXS_FORMULA		1
#define NEXS_LABEL		2

/* status values for dialog callback */

#define	NEXS_DIALOG_BUSY	0
#define	NEXS_DIALOG_OK		1
#define	NEXS_DIALOG_CANCELED	2

/******************************************************************************/

/* "Default Sheet Characteristics" */

#define	OFF			0
#define	ON			1

#define	JUST_LEFT		0
#define	JUST_CENTER		1
#define	JUST_RIGHT		2

#define	MOVE_OFF		0
#define	MOVE_UP			1
#define	MOVE_DOWN		2
#define	MOVE_RIGHT		3
#define	MOVE_LEFT		4

#define	COL_WIDTH		(1<<0)
#define	ITER_LIMIT		(1<<1)
#define	TEXT_JUST		(1<<2)
#define	AUTO_MOVE		(1<<3)
#define	AUTO_ROUND		(1<<4)
#define	ROUND_PLACES		(1<<5)
#define	AUTO_RECALC		(1<<6)
#define	CONSTRAINT_CHECK	(1<<7)
#define	CELL_PROTECT		(1<<8)
#define	PRINT_TITLES		(1<<9)
#define	GRID_ENABLED		(1<<10)
#define	ALL			(~0)

typedef struct	{
		unsigned int col_width,
			     iter_limit,
			     text_just,
			     auto_move,
			     auto_round,
			     round_places,
			     auto_recalc,
			     constraint_check,
			     cell_protect,
			     print_titles,
			     grid_enabled;
		} SheetDefaults;

/******************************************************************************/

/* "Default Cell Characteristics" */

#define	FORMAT			(1<<0)
#define	FONT			(1<<1)
#define	PLACES			(1<<2)


typedef struct	{
		unsigned int format,
			     font,
			     places;
		} CellDefaults;

/******************************************************************************/

/* "Postscript Output Characteristics" */

#define	PAGE_WIDTH		(1<<0)
#define	PAGE_HEIGHT		(1<<1)
#define	LEFT_MARGIN		(1<<2)
#define	RIGHT_MARGIN		(1<<3)
#define	TOP_MARGIN		(1<<4)
#define	BOTTOM_MARGIN		(1<<5)
#define	FONT_SIZE		(1<<6)
#define	FONT_FAMILY		(1<<7)
#define	FONT_PATH		(1<<8)
#define	LANDSCAPE		(1<<9)
#define	AUTO_PAGINATE		(1<<10)
#define	AUTO_SHRINK		(1<<11)
#define	AUTO_CENTER		(1<<12)

/* Font Families */

#define	Courier			0
#define	Helvetica		1
#define	TimesRoman		2
#define	AvantGarde		3
#define	LubalinGraph		4
#define	Souvenir		5

typedef struct	{
		float		page_width,	/* units of float parameters */
				page_height,	/* are printer's points      */
				left_margin,
				right_margin,
				top_margin,
				bottom_margin,
				font_size;
		unsigned int	font_family,
				auto_shrink,
				auto_paginate,
				auto_center,
				landscape;
		char		*font_path;
		} PSDefaults;

/******************************************************************************/

/* "Printer Characteristics */

#define	FILE_DISPOSITION	(1<<0)
#define	PRINT_COMMAND		(1<<1)

/* File Dispositions */

#define	SEND_TO_PRINTER		0
#define	SEND_TO_FILE		1
#define	SEND_TO_BOTH		2

typedef struct	{
		unsigned int	file_disp;
		char		*print_cmd;
		} PrintDefaults;

/******************************************************************************/

/* "Import Characteristics */

#define	ONE_WORD_PER_COL	(1<<0)	/* one word per column on import      */
#define	WHITESPACE_THRESHOLD	(1<<1)	/* white space threshold              */
#define	IMPORTCHAR_RESET	(1<<2)	/* reset import/export charactistics  */
#define	EXPORTCHAR_AUTOESCAPE	(1<<3)	/* autoescape LaTeX special chars     */

/******************************************************************************/

/* File types for nexs_print() */

#define		PRINT_ASCII	1
#define		PRINT_PS	2

/* File types for nexs_import()/nexs_export() */

#define		WKS_FILE	1
#define		WK1_FILE	2
#define		TEXT_FILE	3	/* plain text                         */
#define		XSC_FILE	4	/* NExS native cell descriptions      */
#define		CSV_FILE	5	/* comma separated values             */
#define		TSV_FILE	6	/* tab separated values               */
#define		TEX_FILE	7	/* LaTeX 2.09 tabular format          */
#define		HTML_FILE	8	/* Netscape (R) compatible HTML table */
#define		TEX2E_FILE	9	/* LaTeX 2e tabular format            */
#define		HTML3_FILE	10	/* HTML 3.0 table                     */

/* modes for nexs_set_titles() and nexs_clear_titles() */

#define		TITLE_ROWS	0
#define		TITLE_COLS	1
#define		TITLE_BOTH	2

/******************************************************************************/

/* direction codes for nexs_sort() */

#define		SORT_ASCENDING	0
#define		SORT_DESCENDING	1

/* modes for nexs_sort() */

#define		SORT_UPDATE	0
#define		SORT_NO_UPDATE	1

/* error codes for nexs_goalseek() */

#define		GS_FORMULA_ERR	-1
#define		GS_VARIABLE_ERR	-2
#define		GS_FP_ERR	-3
#define		GS_TARGET_ERR	-4
#define		GS_BRACKET_ERR	-5
#define		GS_ITER_ERR	-6

/* search direction codes for nexs_find() */

#define		FIND_NEXT	0
#define		FIND_PREV	1

/* search order codes for nexs_find() */

#define		FIND_BY_COL	0
#define		FIND_BY_ROW	1

/* condition type codes for nexs_find() and nexs_extract() */

#define		FIND_REGEX	0
#define		FIND_NUMERIC	1

/******************************************************************************/

/* Structures and definitions used for Graph Connections */

/* Graph Types */

#define	GT_LINE		0
#define	GT_HISTOGRAM	1
#define	GT_BAR		2
#define	GT_PIE		3
#define	GT_SURFACE	4
#define	GT_CONTOUR	5
#define	GT_POLAR	6
#define	GT_HILO		7
#define GT_STACKLINE    8
#define GT_STACKBAR     9
#define GT_BOX          10
#define GT_WESTHSE      11

/* Surface Shade Types */

#define	SS_NONE		0
#define	SS_LIGHT_SRC	1
#define	SS_Z_AXIS	2

/* Marker Types */

#define	MT_NONE		0
#define	MT_SQUARE	1
#define	MT_CIRCLE	2
#define	MT_TRIANGLE	3
#define	MT_DIAMOND	4
#define	MT_INV_TRIANGLE	5

/* Line Styles */

#define	LS_SOLID	0
#define	LS_DOTTED	1
#define	LS_DASHED	2
#define	LS_LONG_DASHED	3
#define	LS_DOT_DASHED	4

/* Segment Types */

#define	ST_NONE		0
#define	ST_LINE		1
#define	ST_SPLINE	2

/* GrGenChar Mask */

#define	GR_TYPE		(1<<0)
#define	SURFACE_SHADE	(1<<1)
#define	RECALC_UPDATE	(1<<2)
#define	DISP_LEGEND	(1<<3)
#define	TITLE_TEXT	(1<<4)
#define	TITLE_FONT	(1<<5)
#define SKEW_DATA	(1<<6)
 
typedef struct	{
		int graph_type;
		int surface_shade;
		int recalc_update;
		int disp_legend;
		char *title_text;
		int title_font;		/* FNT_NORMAL, etc. */
		int skew_data;
		} GrGenChar;

/* GrAxisChar Mask */

#define	LABEL_TEXT	(1<<0)
#define	LABEL_FONT	(1<<1)
#define	NUM_TICKS	(1<<2)
#define	AXIS_MIN	(1<<3)
#define	AXIS_MAX	(1<<4)
#define	LOG_SCALE	(1<<5)
#define	AUTO_TICKS	(1<<6)
#define	AUTO_SCALE	(1<<7)
#define	OUT_TICKS	(1<<8)
#define	EXPLODED_AXIS	(1<<9)
#define	GRID_LINES	(1<<10)

/* Axis Specifiers */

#define X_AXIS		0
#define Y_AXIS		1
#define Z_AXIS		2
#define AUX_Y_AXIS	3

typedef	struct	{
		char *label_text;
		int label_font;
		int num_ticks;
		double axis_min;
		double axis_max;
		int log_scale;
		int auto_ticks;
		int auto_scale;
		int out_ticks;
		int exploded_axis;
		int grid_lines;
		} GrAxisChar;

/* XS2GrDataChar Mask */

#define	X_RANGE		(1<<0)
#define	Y_RANGE		(1<<1)
#define	Z_RANGE		(1<<2)
#define	MARKER_TYPE	(1<<3)
#define	LINE_STYLE	(1<<4)
#define	SEGMENT_TYPE	(1<<5)
#define	LEGEND_TEXT	(1<<6)
#define	DATA_DISABLE	(1<<7)

typedef struct	{
		char *x_range, *y_range, *z_range;
		int marker_type;
		int line_style;
		int segment_type;
		char *legend_text;
		int data_disable;
		} XS2GrDataChar;

/* GrRotate3D Mask */

#define THETA		(1<<0)
#define PHI		(1<<1)

typedef struct	{
		double theta;
		double phi;
		} GrRotate3D;

/* Graph Print Types */

#define	GR_EPS		0	/* Encapsulated PostScript */
#define	GR_IDRAW_PS	1	/* Idraw compatible PostScript */

/* GrGeometry Mask */

#define	GR_X_POS	(1<<0)
#define	GR_Y_POS	(1<<1)
#define	GR_WIDTH	(1<<2)
#define	GR_HEIGHT	(1<<3)

typedef struct	{
		int x;
		int y;
		int width;
		int height;
		} GrGeometry;

/******************************************************************************/

/* Structure used to buffer information returned by nexs_get_numbers */

typedef struct	{
		int row, col;
		Number value;
		} XSValBuf;

/******************************************************************************/

/* Action parameters for nexs_set_action_notify() */

#define	ACTION_NEW_SHEET	0
#define	ACTION_GRAPH_PICK	1

/* Structures to provide detail information for actions */

typedef struct	{
		int gid;	/* graph ID */
		int xr, xc;	/* cell coordinates of x-pick */
		int yr, yc;	/* cell coordinates of y-pick */
		} GraphPickDetail;

typedef union	{
		GraphPickDetail gp;
		} ActionDetail;

/******************************************************************************/

/* Structure and mask definition used by nexs_cell_attributes */

/* NExS font family codes */

#define NEXS_FONT_FAMILY_DEFAULT	0
#define NEXS_FONT_FAMILY_COURIER	1
#define NEXS_FONT_FAMILY_ROMAN		2
#define NEXS_FONT_FAMILY_HELVETICA	3
#define NEXS_FONT_FAMILY_SOUVENIR	4
#define NEXS_FONT_FAMILY_LUBALIN	5

/* NExS font size codes */

#define NEXS_FONT_SIZE_DEFAULT		0
#define NEXS_FONT_SIZE_8PT		1
#define NEXS_FONT_SIZE_10PT		2
#define NEXS_FONT_SIZE_12PT		3
#define NEXS_FONT_SIZE_14PT		4
#define NEXS_FONT_SIZE_18PT		5
#define NEXS_FONT_SIZE_24PT		6

/* NExS font style codes */

#define NEXS_FONT_STYLE_DEFAULT		0
#define NEXS_FONT_STYLE_NORMAL		1
#define NEXS_FONT_STYLE_BOLD		2
#define NEXS_FONT_STYLE_ITALIC		3
#define NEXS_FONT_STYLE_BOLDITALIC	4

/* NExS format codes.  (If NEXS_FMT_DEFAULT is selected, the default	*/
/* number of decimal places will be used.  Otherwise, "places" may be	*/
/* in the range 0 to 15.						*/

#define NEXS_FMT_DEFAULT		0
#define NEXS_FMT_FLOAT			1
#define NEXS_FMT_FIXED			2
#define NEXS_FMT_GEN			3
#define NEXS_FMT_DOLLARS		4
#define NEXS_FMT_COMMA			5
#define NEXS_FMT_HEX			6
#define NEXS_FMT_LOGIC			7
#define NEXS_FMT_DAY_MONTH_YEAR		8
#define NEXS_FMT_DAY_MONTH		9
#define NEXS_FMT_MONTH_YEAR		10
#define NEXS_FMT_DATE			11
#define NEXS_FMT_HIDDEN			12
#define NEXS_FMT_TIME			13
#define NEXS_FMT_PERCENT		14
#define NEXS_FMT_TEXT			15
#define NEXS_FMT_CURRENCY_1		16
#define NEXS_FMT_CURRENCY_2		17
#define NEXS_FMT_CURRENCY_3		18
#define NEXS_FMT_CURRENCY_4		19
#define NEXS_FMT_INTL_DATE		20
#define NEXS_FMT_ISO8061_DATE		21


/* NExS cell color codes.  Valid NExS color map values are in the	*/
/* range 1 to 15.  (0 selects default colors.)				*/

#define NEXS_FG_DEFAULT			0
#define NEXS_BG_DEFAULT			0

/* NExS justification codes */

#define NEXS_JUST_DEFAULT		0
#define NEXS_JUST_LEFT			1
#define NEXS_JUST_RIGHT			2
#define NEXS_JUST_CENTER		3
#define NEXS_JUST_AUTO			4

/* NExS protection level codes.  Level 1 is unprotected. */

#define NEXS_DEFAULT_PROTECT		0
#define NEXS_PROTECT_LEVEL_1		1
#define NEXS_PROTECT_LEVEL_2		2

/* NExS word wrap codes. */

#define NEXS_WRAP_DEFAULT		0
#define NEXS_WRAP_OFF			1
#define NEXS_WRAP_ON			2

/* NExS underline codes. */

#define NEXS_UNDERLINE_DEFAULT		0
#define NEXS_UNDERLINE_OFF		1
#define NEXS_UNDERLINE_ON		2

/* NExS Cell Attribute Mask Values */

#define NEXS_FONT_FAMILY_MASK		(1<<0)
#define NEXS_FONT_SIZE_MASK		(1<<1)
#define NEXS_FONT_STYLE_MASK		(1<<2)
#define NEXS_COLOR_MASK			(1<<3)
#define NEXS_JUST_MASK			(1<<4)
#define NEXS_PROTECT_MASK		(1<<5)
#define NEXS_WRAP_MASK			(1<<6)
#define NEXS_UNDERLINE_MASK		(1<<7)
#define NEXS_FORMAT_MASK		(1<<8)
#define NEXS_ALL_MASK			-1

/* NExS Cell Attributes Structure */

typedef struct	{
		int font_family,
		    font_size,
		    font_style,
		    fg,
		    bg,
		    just,
		    protect,
		    wrap,
		    underline,
		    format,
		    places;
		} CellAttributes;


/* NExS Graph Dataset Attributes codes */

/* Marker Type */

#define NEXS_MT_DEFAULT		0
#define NEXS_MT_NONE		1
#define NEXS_MT_SQUARE		2
#define NEXS_MT_CIRCLE		3
#define NEXS_MT_TRIANGLE	4
#define NEXS_MT_DIAMOND		5
#define NEXS_MT_INV_TRI		6

/* Line Style */

#define NEXS_LS_DEFAULT		0
#define NEXS_LS_NONE		1
#define NEXS_LS_SOLID		2
#define NEXS_LS_DOTTED		3
#define NEXS_LS_FINE_DASH	4
#define NEXS_LS_LONG_DASH	5
#define NEXS_LS_DOT_DASH	6

/* Line Width */

#define NEXS_LW_DEFAULT		0
#define NEXS_LW_THIN		1
#define NEXS_LW_MEDIUM		2
#define NEXS_LW_THICK		4

/* Point Connection Style */

#define NEXS_CS_LINE		0
#define NEXS_CS_SPLINE		1
#define NEXS_CS_BAR		2

/* Fill Style */

#define NEXS_FS_DEFAULT		0
#define NEXS_FS_NONE		1
#define NEXS_FS_SOLID		2
#define NEXS_FS_GRAY_0		3
#define NEXS_FS_GRAY_25		4
#define NEXS_FS_GRAY_50		5
#define NEXS_FS_GRAY_75		6
#define NEXS_FS_GRAY_100	7

/* Shading Style */

#define NEXS_SS_NONE		0
#define NEXS_SS_LIGHT_SOURCE	1
#define NEXS_SS_Z_HEIGHT	2

/* Colors */

#define NEXS_COLOR_DEFAULT	0
/* The color palette range is 1-15 */

/* NExS Graph Dataset Attributes Mask Values */

#define NEXS_GDATA_X_RANGE		(1<<0)
#define NEXS_GDATA_Y_RANGE		(1<<1)
#define NEXS_GDATA_Z_RANGE		(1<<2)
#define NEXS_GDATA_YLO_RANGE		(1<<3)
#define NEXS_GDATA_YHI_RANGE		(1<<4)
#define NEXS_GDATA_ANNOT_RANGE		(1<<5)
#define NEXS_GDATA_LEGEND_TEXT		(1<<6)
#define NEXS_GDATA_LEGEND_COLOR		(1<<7)
#define NEXS_GDATA_LEGEND_FAMILY	(1<<8)
#define NEXS_GDATA_LEGEND_STYLE		(1<<9)
#define NEXS_GDATA_LEGEND_SIZE		(1<<10)
#define NEXS_GDATA_ANNOT_COLOR		(1<<11)
#define NEXS_GDATA_ANNOT_FAMILY		(1<<12)
#define NEXS_GDATA_ANNOT_STYLE		(1<<13)
#define NEXS_GDATA_ANNOT_SIZE		(1<<14)
#define NEXS_GDATA_MRK			(1<<15)
#define NEXS_GDATA_LINESTYLE		(1<<16)
#define NEXS_GDATA_LINEWIDTH		(1<<17)
#define NEXS_GDATA_LINECOLOR		(1<<18)
#define NEXS_GDATA_CONNECTION		(1<<19)
#define NEXS_GDATA_DATASET_ONOFF	(1<<20)
#define NEXS_GDATA_FILLSTYLE		(1<<21)
#define NEXS_GDATA_FILLCOLOR		(1<<22)
#define NEXS_GDATA_HISTOGRAM_ONOFF	(1<<23)
#define NEXS_GDATA_SHADING		(1<<24)
#define NEXS_GDATA_YVARLO_RANGE		(1<<25)
#define NEXS_GDATA_YVARHI_RANGE		(1<<26)
#define NEXS_GDATA_YMED_RANGE		(1<<27)
#define NEXS_GDATA_ALL			(~0)

/* NExS Graph Dataset Attributes Structure */

typedef struct	{
		char x_range[MAX_RANGE_TEXT],		/* X Data       */
		     y_range[MAX_RANGE_TEXT],		/* Y Data       */
		     z_range[MAX_RANGE_TEXT],		/* Z Data       */
		     ylo_range[MAX_RANGE_TEXT],		/* Lo Data      */
		     yhi_range[MAX_RANGE_TEXT],		/* Hi Data      */
		     yvarlo_range[MAX_RANGE_TEXT],	/* Lo Variance Data */
		     yvarhi_range[MAX_RANGE_TEXT],	/* Hi Variance Data */
		     ymed_range[MAX_RANGE_TEXT],	/* Median Data  */
		     annot_range[MAX_RANGE_TEXT],	/* Annotation   */
		     legend_text[MAX_TEXT];		/* Legend       */
		int  legend_color,			/* Legend Color */
		     legend_family,			/* Legend Font  */
		     legend_style,			/* Legend Style */
		     legend_size,			/* Legend Size  */
		     annot_color,			/* N/A          */
		     annot_family,			/* N/A          */
		     annot_style,			/* N/A          */
		     annot_size,			/* N/A          */
		     mrk,				/* Marker       */
		     linestyle,				/* Line Style   */
		     linewidth,				/* Line Width   */
		     linecolor,				/* Line Color   */
		     connection,			/* Segment Type */
		     dataset_onoff,			/* Disable      */
		     fillstyle,				/* Fill Style   */
		     fillcolor,				/* Fill Color   */
		     histogram_onoff,			/* Histogram    */
		     shading;				/* Shading      */
		} NEXSGrDataChar;


/* NExS Currency Style Options */

/* Currency digits separator style (CurrencyStyle.sep) */

#define NEXS_CURR_SEP_1			0	/* 1,234,567.89 */
#define NEXS_CURR_SEP_2			1	/* 1.234.567,89 */
#define NEXS_CURR_SEP_3			2	/* 1 234 567.89 */
#define NEXS_CURR_SEP_4			3	/* 1 234 567,89 */

/* How negative currency values are displayed (CurrencyStyle.neg) */

#define NEXS_CURR_NEG_PAREN		0	/* (1,234.00 US) */
#define NEXS_CURR_NEG_MINUS		1	/* -1,234.00 US  */

/* Where the tag is displayed (CurrencyStyle.tag_loc) */

#define NEXS_CURR_TAG_IS_PREFIX		0	/* US 1,234.00 */
#define NEXS_CURR_TAG_IS_SUFFIX		1	/* 1,234.00 US */

/* NExS Currency Style Options Mask Values */

#define NEXS_CURRENCY_SEP_MASK          (1<<0)
#define NEXS_CURRENCY_NEG_MASK          (1<<1)
#define NEXS_CURRENCY_TAG_LOC_MASK      (1<<2)
#define NEXS_CURRENCY_TAG_MASK          (1<<3)

typedef struct {
    int sep;				/* digits separator */
    int neg;				/* negative values indicator */
    int tag_loc;			/* prefix or suffix */
    char tag[8];			/* the currency tag itself */
    } CurrencyStyle;



/******************************************************************************/
/* NExS I/O declarations */
/*
#include	<stdio.h>
#include	<string.h>
#include	<assert.h>
#include        "nexs.h"
#include        "malloc.h"
*/
#include	<setjmp.h>

#define NEXS_IO_PARAM			0
#define NEXS_IO_GET_NUM_ROWS		1
#define NEXS_IO_GET_NUM_COLS		2
#define NEXS_IO_GET_LENGTH		3

#define	NEXS_IO_VOID			0
#define	NEXS_IO_INTEGER			1
#define	NEXS_IO_FLOAT			2
#define	NEXS_IO_DOUBLE			3
#define	NEXS_IO_CHARPTR			4

extern jmp_buf	nexs_io_env;

typedef enum { UNSUPPORTED_OPERAND_TYPE,
		DIMENSION_MISMATCH_NUMBER,
		DIMENSION_MISMATCH_CELL,
		DIMENSION_MISMATCH_VECTOR,
		DIMENSION_MISMATCH_ROWS,
		DIMENSION_MISMATCH_COLS,
		MATRIX_RANGE_FOR_VECTOR,
		NON_SCALAR_RANGE,
		STORING_INTO_NUMBER,
		STORING_INTO_STRING,
		OUT_OF_STRING_MEMORY,
		FREEING_A_NUMBER,
		FREEING_A_STRING
		} nexs_io_errors;

/******************************************************************************/



/******************************************************************************/

/* Global variables created by the Connection Library */

extern int	nexs_row, nexs_col,  /* cell from which function was invoked */
		nexs_recalc_flags,   /* current recalc flags in effect       */
		nexs_tag,	     /* tag for remote functions and buttons */
		nexs_port,	     /* port from which function was invoked */
		nexs_conv_warn;	     /* floating point conversion problem    */

extern char	nexs_sbuf[];

/******************************************************************************/


/* Public function prototypes for libnexs.a */

#if NeedFunctionPrototypes

extern int nexs_connect_init(Display *dp);

extern int nexs_set_display(int displayID);

extern int nexs_get_display(void);

extern int nexs_remove_display(int displayID);

extern int nexs_new_display(Display *dp);

extern int nexs_establish_connection(char *name, int timeout);

extern int nexs_get_string(int port, int row, int col, char **str);

extern int nexs_get_number(int port, int row, int col, double *value);

extern int nexs_get_value(int port,
			int row, int col, double *value, char **str);

extern int nexs_get_formula(int port, int row, int col, char **str);

extern int nexs_get_location(int port, int *rowp, int *colp);

extern void nexs_store_number(int port, int row, int col, double value);

extern void nexs_store_formatted_number(int port,
	int row, int col, double value, int fnt_code, int fmt_code, int places);

extern void nexs_store_tagged_number(int port, int row, int col, double value);

extern void nexs_store_label(int port, int row, int col, char *s);

extern void nexs_store_tagged_string(int port, int row, int col, char *s);

extern void nexs_store_formula(int port, int row, int col, char *s);

extern void nexs_display_message(int port, char *s);

extern void nexs_goto(int port, int row, int col);

extern void nexs_moveto(int port, int row, int col);

extern void nexs_close_connection(int port);

extern int nexs_open_connection(char *name, int waitflag);

extern long nexs_create_menu(int port, char *name);

extern long nexs_create_button(int port,
		long int menuID, char *name, void (*callback)(), int tag);

extern int nexs_message_handler(XEvent *event);

extern void nexs_set_close_notify(void (*fnp)());

extern int nexs_recalc(int port, int force, int n, Range *ranges);

extern void nexs_delete_menu(int port, long int menuID);

extern void nexs_delete_button(int port, long int menuID, long int buttonID);

extern void nexs_menu_sensitivity(int port, long int menuID, int sensitivity);

extern void nexs_button_sensitivity(int port,
			long int menuID, long int buttonID, int sensitivity);

extern void nexs_menu_label(int port, long int menuID, char *label);

extern void nexs_button_label(int port,
			long int menuID, long int buttonID, char *label);

extern void nexs_dispatcher(void (*callback)());

extern void nexs_flush(void);

extern int nexs_flush_port(int port);

extern int nexs_get_range(int port, int *r0p, int *c0p, int *r1p, int *c1p);

extern int nexs_get_selected(int port, int *r0p, int *c0p, int *r1p, int *c1p);

extern void nexs_install_function(int port,
			char *name, Operand (*callback)(), int tag);

extern int nexs_dialog(int port, char *prompt, char *def, void (*callback)());

extern int nexs_dialog_no_echo(int port,
			char *prompt, char *def, void (*callback)());

extern int nexs_get_input(int port, char *prompt, char *buf, int maxchars);

extern int nexs_evaluate_cell(int port,
			int row, int col, double *value, char **str);

extern int nexs_evaluate_constraint(int port, int row, int col);

extern int nexs_recalc_constraints(int port);

extern int nexs_find_label(int port, char *label, int *rp, int *cp);

extern int nexs_find_label_in_range(int port,
		char *label, int r0, int c0, int r1, int c1, int *rp, int *cp);

extern int nexs_define_name(int port, Range *rangep, char *name);

extern int nexs_resolve_name(int port, Range *rangep, char *name);

extern void nexs_delete_name(int port, char *name);

extern int nexs_get_extent(int port,
			int *rminp, int *cminp, int *rmaxp, int *cmaxp);

extern int nexs_scroll_range(int port,
			int r0, int c0, int r1, int c1, int direction);

extern int nexs_evaluate_expr(int port, char *expr, double *value, char **str);

extern int nexs_read_file(int port, char *name);

extern int nexs_write_file(int port, char *name);

extern void nexs_user_lock(int port, int lockflag);

extern Window nexs_window(int port);

extern Display *nexs_display(int port);

extern Display *nexs_display_from_ID(int displayID);

extern char *nexs_version(int port);

extern int nexs_test_connection(int port);

extern int nexs_inquire_connections(int port,
				int *status_list, char (*name_list)[11]);

extern int nexs_send_message(int port, int remote_port, char *message);

extern void nexs_set_receiver(void (*fnp)());

extern int nexs_message_wait(int port, int remote_port, char *msg_buf);

extern int nexs_sheet_defaults(int port, SheetDefaults *defaults, int mask);

extern int nexs_cell_defaults(int port, CellDefaults *defaults, int mask);

extern int nexs_ps_char(int port, PSDefaults *defaults, int mask);

extern int nexs_print_char(int port, PrintDefaults *defaults, int mask);

extern int nexs_print(int port, int filetype, Range range, char *filename);

extern int nexs_import(int port, int filetype, char *filename, int r, int c);

extern int nexs_export(int port, int filetype, char *filename, Range range);

extern int nexs_copy_range(int port, int vflag, Range src, Range dst);

extern int nexs_erase_range(int port, Range range);

extern int nexs_move_range(int port, Range src, int r_dst, int c_dst);

extern int nexs_insert_rows(int port, int row, int num_rows);

extern int nexs_insert_cols(int port, int col, int num_cols);

extern int nexs_delete_rows(int port, int start_row, int end_row);

extern int nexs_delete_cols(int port, int start_col, int end_col);

extern int nexs_clear_sheet(int port);

extern int nexs_set_format(int port, int fmt_code, int places, Range range);

extern int nexs_set_font(int port, int fnt_code, Range range);

extern int nexs_protect(int port, int flag, Range range);

extern int nexs_set_titles(int port, int r, int c, int mode);

extern int nexs_clear_titles(int port, int mode);

extern int nexs_goalseek(int port, int vr, int vc, int tr, int tc,
	double target, double *x1, double *x2, int maxit, double tolerance);

extern int nexs_sort(int port,
		int dir, int mode, Range range, int keycol1, int keycol2);

extern int nexs_find(int port, int order, int dir,
	Range range, char *condition, int numeric, int *find_r, int *find_c);

extern int nexs_extract(int port, Range range, int keycol,
			char *condition, int numeric, int r_dst, int c_dst);

extern Window nexs_get_acceptor(int displayID);

extern char *nexs_get_name(int displayID, Window nexs_window);

extern int nexs_scan_display(int displayID,
				Window **nexs_windows, int *num_windows);

extern int nexs_ggen_char(int port, int gid, GrGenChar *gchar, int mask);

extern int nexs_axis_char(int port,
			int gid, int axis, GrAxisChar *axischar, int mask);

extern int xs2_gdata_char(int port,
			int gid, int dataset, XS2GrDataChar *dchar, int mask);

extern int nexs_rotate_graph(int port, int gid, GrRotate3D *grotate, int mask);

extern int nexs_graph_geometry(int port, int gid, GrGeometry *ggeom, int mask);

extern int nexs_redraw_graph(int port, int gid);

extern int nexs_name_graph(int port, int gid, char *name);

extern int nexs_duplicate_graph(int port, int gid, int *new_gid);

extern int nexs_delete_graph(int port, int gid);

extern int nexs_print_graph(int port, int gid, int type, char *name);

extern int nexs_cancel_graph(int port, int gid);

extern int nexs_display_graph(int port, int gid);

extern int nexs_new_graph(int port, int *new_gid);

extern Window nexs_graph_window(int port, int gid);

extern Window nexs_top_window(int port);

extern int nexs_get_numbers(int port,
	Range range, int start_row, int start_col, XSValBuf *vbuf, int *count);

extern int nexs_set_width(int port, int c0, int c1, int width, int deflt);

extern int nexs_get_width(int port, int c, int *width);

extern int nexs_import_characteristics(int port,
	int *one_word_per_col, int *ws_threshold, int *autoesc, int mask);

extern int nexs_insert_pagebreak(int port, int row);

extern void nexs_set_action_callback(void (*callback)(int, int, ActionDetail *));

extern int nexs_set_action_notify(int port, int action, int flag);

extern int nexs_queue_callback(int port, void (*callback)(), int tag);

extern int mgr_connect(int displayID, Window nexs_window, char *name);

extern int mgr_set_nexs_notify(void (*fnp)());

extern int mgr_manage_display(int displayID);

extern int mgr_unmanage_display(int displayID);

extern int mgr_set_acceptor(int displayID, Window nexs_window);

/* start of NExS I/O routines */
void nexs_io_error(nexs_io_errors err_num);
void nexs_io_error_msg(char *msg);
char* rctotext(int c,int r);
void nexs_io_op_error_msg(Operand* op,char* format,...);
int nexs_io_get_op_size(Operand *operand,int selector);
int nexs_io_open_nexs_connection(char *connection_name,int wait_before_failure);
void nexs_io_operand_slice(Operand *src_op,int r0,int c0,int r1,int c1,Operand *dst_op);

Number nexs_io_get_scalar(Operand *operand);
void nexs_io_store_scalar(Number s,Operand *range);

char *nexs_io_get_str(Operand *operand);
void nexs_io_fre_string(char *s,Operand *range);
void nexs_io_str_string(char *s,Operand *range);

void nexs_io_free_ivector (int  *v,int nl,int nh);
void nexs_io_free_imatrix (int  **m,int nrl,int nrh,int ncl,int nch);
int  * nexs_io_ivector (int nl,int nh);
int  * nexs_io_ivector_realloc (int  *v,int nl,int nh,int new_nl,int new_nh);
int  ** nexs_io_imatrix (int nrl,int nrh,int ncl,int nch);
int  * nexs_io_alo_int_vector (Operand *range,int lo,int hi);
int  ** nexs_io_alo_int_matrix (Operand *range,int rlo,int rhi,int clo,int chi);
int  * nexs_io_get_int_vector (Operand *range,int lo,int hi);
int  ** nexs_io_get_int_matrix (Operand *range,int rlo,int rhi,int clo,int chi);
void nexs_io_str_int_vector (int  *v,Operand *range,int lo,int hi);
void nexs_io_str_int_matrix (int  **m,Operand *range,int rlo,int rhi,int clo,int chi);
void nexs_io_fre_int_vector (int  *v,Operand *range,int lo,int hi);
void nexs_io_fre_int_matrix (int  **m,Operand *range,int rlo,int rhi,int clo,int chi);

void nexs_io_free_vector (float  *v,int nl,int nh);
void nexs_io_free_matrix (float  **m,int nrl,int nrh,int ncl,int nch);
float  * nexs_io_vector (int nl,int nh);
float  * nexs_io_vector_realloc (float  *v,int nl,int nh,int new_nl,int new_nh);
float  ** nexs_io_matrix (int nrl,int nrh,int ncl,int nch);
float  * nexs_io_alo_float_vector (Operand *range,int lo,int hi);
float  ** nexs_io_alo_float_matrix (Operand *range,int rlo,int rhi,int clo,int chi);
float  * nexs_io_get_float_vector (Operand *range,int lo,int hi);
float  ** nexs_io_get_float_matrix (Operand *range,int rlo,int rhi,int clo,int chi);
void nexs_io_str_float_vector (float  *v,Operand *range,int lo,int hi);
void nexs_io_str_float_matrix (float  **m,Operand *range,int rlo,int rhi,int clo,int chi);
void nexs_io_fre_float_vector (float  *v,Operand *range,int lo,int hi);
void nexs_io_fre_float_matrix (float  **m,Operand *range,int rlo,int rhi,int clo,int chi);

void nexs_io_free_dvector (double  *v,int nl,int nh);
void nexs_io_free_dmatrix (double  **m,int nrl,int nrh,int ncl,int nch);
double  * nexs_io_dvector (int nl,int nh);
double  * nexs_io_dvector_realloc (double  *v,int nl,int nh,int new_nl,int new_nh);
double  ** nexs_io_dmatrix (int nrl,int nrh,int ncl,int nch);
double  * nexs_io_alo_double_vector (Operand *range,int lo,int hi);
double  ** nexs_io_alo_double_matrix (Operand *range,int rlo,int rhi,int clo,int chi);
double  * nexs_io_get_double_vector (Operand *range,int lo,int hi);
double  ** nexs_io_get_double_matrix (Operand *range,int rlo,int rhi,int clo,int chi);
void nexs_io_str_double_vector (double  *v,Operand *range,int lo,int hi);
void nexs_io_str_double_matrix (double  **m,Operand *range,int rlo,int rhi,int clo,int chi);
void nexs_io_fre_double_vector (double  *v,Operand *range,int lo,int hi);
void nexs_io_fre_double_matrix (double  **m,Operand *range,int rlo,int rhi,int clo,int chi);

char **svector(int nl,int nh);
char ***smatrix(int nrl,int nrh,int ncl,int nch);
void free_svector(char **v,int nl,int nh);
void free_smatrix(char ***m,int nrl,int nrh,int ncl,int nch);
char*  * nexs_io_alo_string_vector (Operand *range,int lo,int hi);
char*  ** nexs_io_alo_string_matrix (Operand *range,int rlo,int rhi,int clo,int chi);
char*  * nexs_io_get_string_vector (Operand *range,int lo,int hi);
char*  ** nexs_io_get_string_matrix (Operand *range,int rlo,int rhi,int clo,int chi);
void nexs_io_str_string_vector (char*  *v,Operand *range,int lo,int hi);
void nexs_io_str_string_matrix (char*  **m,Operand *range,int rlo,int rhi,int clo,int chi);
void nexs_io_fre_string_vector (char*  *v,Operand *range,int lo,int hi);
void nexs_io_fre_string_matrix (char*  **m,Operand *range,int rlo,int rhi,int clo,int chi);

#else

extern int nexs_connect_init();

extern int nexs_set_display();

extern int nexs_get_display();

extern int nexs_remove_display();

extern int nexs_new_display();

extern int nexs_establish_connection();

extern int nexs_get_string();

extern int nexs_get_number();

extern int nexs_get_value();

extern int nexs_get_formula();

extern int nexs_get_location();

extern void nexs_store_number();

extern void nexs_store_formatted_number();

extern void nexs_store_tagged_number();

extern void nexs_store_label();

extern void nexs_store_tagged_string();

extern void nexs_store_formula();

extern void nexs_display_message();

extern void nexs_goto();

extern void nexs_moveto();

extern void nexs_close_connection();

extern nexs_open_connection();

extern long nexs_create_menu();

extern long nexs_create_button();

extern nexs_message_handler();

extern void nexs_set_close_notify();

extern nexs_recalc();

extern void nexs_delete_menu();

extern void nexs_delete_button();

extern void nexs_menu_sensitivity();

extern void nexs_button_sensitivity();

extern void nexs_menu_label();

extern void nexs_button_label();

extern void nexs_dispatcher();

extern int nexs_flush();

extern int nexs_flush_port();

extern int nexs_get_range();

extern int nexs_get_selected();

extern void nexs_install_function();

extern int nexs_dialog();

extern int nexs_dialog_no_echo();

extern int nexs_get_input();

extern int nexs_evaluate_cell();

extern int nexs_evaluate_constraint();

extern int nexs_recalc_constraints();

extern int nexs_find_label();

extern int nexs_find_label_in_range();

extern int nexs_define_name();

extern int nexs_resolve_name();

extern void nexs_delete_name();

extern int nexs_get_extent();

extern int nexs_scroll_range();

extern int nexs_evaluate_expr();

extern int nexs_read_file();

extern int nexs_write_file();

extern void nexs_user_lock();

extern Window nexs_window();

extern Display *nexs_display();

extern Display *nexs_display_from_ID();

extern char *nexs_version();

extern int nexs_test_connection();

extern int nexs_inquire_connections();

extern int nexs_send_message();

extern void nexs_set_receiver();

extern int nexs_message_wait();

extern int nexs_sheet_defaults();

extern int nexs_cell_defaults();

extern int nexs_ps_char();

extern int nexs_print_char();

extern int nexs_print();

extern int nexs_import();

extern int nexs_export();

extern int nexs_copy_range();

extern int nexs_erase_range();

extern int nexs_move_range();

extern int nexs_insert_rows();

extern int nexs_insert_cols();

extern int nexs_delete_rows();

extern int nexs_delete_cols();

extern int nexs_clear_sheet();

extern int nexs_set_format();

extern int nexs_set_font();

extern int nexs_protect();

extern int nexs_set_titles();

extern int nexs_clear_titles();

extern int nexs_goalseek();

extern int nexs_sort();

extern int nexs_find();

extern int nexs_extract();

extern Window nexs_get_acceptor();

extern char *nexs_get_name();

extern int nexs_scan_display();

extern int nexs_ggen_char();

extern int nexs_axis_char();

extern int xs2_gdata_char();

extern int nexs_rotate_graph();

extern int nexs_graph_geometry();

extern int nexs_redraw_graph();

extern int nexs_name_graph();

extern int nexs_duplicate_graph();

extern int nexs_delete_graph();

extern int nexs_print_graph();

extern int nexs_cancel_graph();

extern int nexs_display_graph();

extern int nexs_new_graph();

extern Window nexs_graph_window();

extern Window nexs_top_window();

extern int nexs_get_numbers();

extern int nexs_set_width();

extern int nexs_get_width();

extern int nexs_import_characteristics();

extern int nexs_insert_pagebreak();

extern void nexs_set_action_callback();

extern int nexs_set_action_notify();

extern int nexs_queue_callback();

extern int mgr_connect();

extern int mgr_set_nexs_notify();

extern int mgr_manage_display();

extern int mgr_unmanage_display();

extern int mgr_set_acceptor();

#endif /* NeedFunctionPrototypes */

/******************************************************************************/


/* Public utility function prototypes */

#if NeedFunctionPrototypes

extern char *col_to_text(unsigned int col);

extern int text_to_col(register char *name);

extern int text_to_rc(char *name, int *row, int *col);

extern int out_of_range(register int r, register int c);

extern int text_to_range(char *name, Range *range);

extern char *range_to_text(register Range *range);

extern int sort_range(register Range *range);

#else

extern char *col_to_text();

extern int text_to_col();

extern int text_to_rc();

extern int out_of_range();

extern int text_to_range();

extern char *range_to_text();

extern int sort_range();

#endif /* NeedFunctionPrototypes */

/******************************************************************************/


/* NExS-only functions (not compatible with XESS Corp. spreadsheet products) */

#if NeedFunctionPrototypes

extern int nexs_cell_attributes(int port, Range range,
			CellAttributes *attr, int mask, int resolve_defaults);

extern int nexs_button_accel(int port, long menuID, long buttonID,
			KeySym mnemonic, char *accel, char *accel_text);

extern int nexs_gdata_char(int port, int gid, int dataset,
				NEXSGrDataChar *dchar, unsigned long mask);

extern int nexs_currency_options(int port, int index,
				CurrencyStyle *cs, unsigned long mask);

#else

extern int nexs_cell_attributes();

extern int nexs_button_accel();

extern int nexs_gdata_char();

extern int nexs_currency_options();

/* start of NExS I/O routines */
void nexs_io_error();
void nexs_io_error_msg();
char* rctotext();
void nexs_io_op_error_msg();
int nexs_io_get_op_size();
int nexs_io_open_nexs_connection();
void nexs_io_operand_slice();

Number nexs_io_get_scalar();
void nexs_io_store_scalar();

char *nexs_io_get_str();
void nexs_io_fre_string();
void nexs_io_str_string();

void nexs_io_free_ivector ();
void nexs_io_free_imatrix ();
int  * nexs_io_ivector ();
int  * nexs_io_ivector_realloc ();
int  ** nexs_io_imatrix ();
int  * nexs_io_alo_int_vector ();
int  ** nexs_io_alo_int_matrix ();
int  * nexs_io_get_int_vector ();
int  ** nexs_io_get_int_matrix ();
void nexs_io_str_int_vector ();
void nexs_io_str_int_matrix ();
void nexs_io_fre_int_vector ();
void nexs_io_fre_int_matrix ();

void nexs_io_free_vector ();
void nexs_io_free_matrix ();
float  * nexs_io_vector ();
float  * nexs_io_vector_realloc ();
float  ** nexs_io_matrix ();
float  * nexs_io_alo_float_vector ();
float  ** nexs_io_alo_float_matrix ();
float  * nexs_io_get_float_vector ();
float  ** nexs_io_get_float_matrix ();
void nexs_io_str_float_vector ();
void nexs_io_str_float_matrix ();
void nexs_io_fre_float_vector ();
void nexs_io_fre_float_matrix ();

void nexs_io_free_dvector ();
void nexs_io_free_dmatrix ();
double  * nexs_io_dvector ();
double  * nexs_io_dvector_realloc ();
double  ** nexs_io_dmatrix ();
double  * nexs_io_alo_double_vector ();
double  ** nexs_io_alo_double_matrix ();
double  * nexs_io_get_double_vector ();
double  ** nexs_io_get_double_matrix ();
void nexs_io_str_double_vector ();
void nexs_io_str_double_matrix ();
void nexs_io_fre_double_vector ();
void nexs_io_fre_double_matrix ();

char **svector();
char ***smatrix();
void free_svector();
void free_smatrix();
char*  * nexs_io_alo_string_vector ();
char*  ** nexs_io_alo_string_matrix ();
char*  * nexs_io_get_string_vector ();
char*  ** nexs_io_get_string_matrix ();
void nexs_io_str_string_vector ();
void nexs_io_str_string_matrix ();
void nexs_io_fre_string_vector ();
void nexs_io_fre_string_matrix ();

#endif /* NeedFunctionPrototypes */

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* NEXS_H */
