/*
 *
 *  NExS Connection Library V2.0
 *  Copyright 1990, 1991, X Engineering Software Systems Corporation
 *  Copyright 1999, GreyTrout Software, Inc.
 *  All rights reserved
 *
 *  file: nexslink.c
 *
 * $Id: nexslink.c,v 1.2 91/08/14 17:24:37 tkm Exp $
 *
 */


#include	<stdio.h>
#include	<stdlib.h>
#include	<X11/Xlib.h>
#include	"nexs.h"

/*
  This program illustrates one way in which NExS Connections can "hot link"
  spreadsheets together.  To use this program, two users on different
  displays running NExS should enable connections and issue the command

	nexslink nodeX

  where "nodeX" is the other user's node name.  Remember that permission
  to use the workstation display must first be granted to the remote node
  using the "xhost" command.
*/

int localport, remoteport, rdID;

char buffer[200];

long menu1ID, button1ID, button2ID, button3ID, button4ID;

void close_notify_fn(port)
int port;
    {
    fprintf(stderr, "Port number %d closed; exiting...\n", port);
    exit(0);
    }

void b1_fn()	/* Send Message */
    {
    *buffer = '\0';
    nexs_get_input(localport, "Please enter message...", buffer, 200);
    if (nexs_test_connection(remoteport))
	{
	nexs_display_message(remoteport, buffer);
	nexs_flush_port(remoteport);
	}
    else
	nexs_display_message(localport, "Remote NExS connection closed");
    }

void b2_fn()	/* Send Values */
    {
    int r, c, r0, c0, r1, c1, status;
    double value;
    char *strptr;

    nexs_get_selected(localport, &r0, &c0, &r1, &c1);
    if (nexs_test_connection(remoteport))
	{
	for (r = r0; r <= r1; r++)
	  for (c = c0; c <= c1; c++)
	    {
	    status = nexs_get_value(localport, r, c, &value, &strptr);
	    switch (status)
		{
		case NEXS_NUMBER:
		    nexs_store_number(remoteport, r, c, value);
		    break;
		case NEXS_STRING:
		    nexs_store_label(remoteport, r, c, strptr);
		    free(strptr);
		    break;
		}
	    }
	nexs_flush_port(remoteport);
	}
    else
	nexs_display_message(localport, "Remote NExS connection closed");
    }

void b3_fn()	/* Send Formulas */
    {
    int r, c, r0, c0, r1, c1, status;
    char *strptr;

    nexs_get_selected(localport, &r0, &c0, &r1, &c1);
    if (nexs_test_connection(remoteport))
	{
	for (r = r0; r <= r1; r++)
	  for (c = c0; c <= c1; c++)
	    {
	    status = nexs_get_formula(localport, r, c, &strptr);
	    switch (status)
		{
		case NEXS_FORMULA:
		    nexs_store_formula(remoteport, r, c, strptr);
		    free(strptr);
		    break;
		case NEXS_LABEL:
		    nexs_store_label(remoteport, r, c, strptr);
		    free(strptr);
		    break;
		}
	    }
	nexs_flush_port(remoteport);
	}
    else
	nexs_display_message(localport, "Remote NExS connection closed");
    }

void exit_fn()
    {
    nexs_close_connection(remoteport);
    nexs_close_connection(localport);
    exit(0);
    }

void usage()
    {
    fprintf(stderr, "usage: nexslink node_name\n");
    exit(0);
    }

int main(argc, argv)
int argc;
char *argv[];
    {
    Display *remote_display;

    if (argc != 2) usage();
    printf("establishing connection...\n");
    nexs_set_close_notify(close_notify_fn);
    localport = nexs_open_connection(argv[1], 10);
    if (localport == 0)
	{
	fprintf(stderr, "could not connect to local NExS\n");
	exit(0);
	}
    sprintf(buffer, "%s:0", argv[1]);
    if ((remote_display = XOpenDisplay(buffer)) == NULL)
	{
	fprintf(stderr, "could not open remote display connection\n");
	exit(0);
	}
    rdID = nexs_new_display(remote_display);
    nexs_set_display(rdID);
    remoteport = nexs_open_connection("NExSLink", 10);
    if (remoteport == 0)
	{
	fprintf(stderr, "could not connect to NExS on remote display\n");
	exit(0);
	}
    menu1ID = nexs_create_menu(localport, argv[1]);
    button1ID = nexs_create_button(localport, menu1ID, "Send Message", b1_fn, 0);
    button2ID = nexs_create_button(localport, menu1ID, "Send Values", b2_fn, 0);
    button3ID = nexs_create_button(localport, menu1ID, "Send Formulas", b3_fn, 0);
    button4ID = nexs_create_button(localport, menu1ID, "Dismiss", exit_fn, 0);

    nexs_set_display(1);
    nexs_dispatcher(NULL);
	 return 0;
    }
