/*
 *
 *  NExS Connection Library
 *  Copyright 1990, 1991, X Engineering Software Systems Corporation
 *  Copyright 1999, GreyTrout Software, Inc.
 *  All rights reserved
 *
 *  file: qscript.c
 *
 */

#include	<stdio.h>
#include	<stdlib.h>
#include <unistd.h>
#include	<ctype.h>
#include	"nexs.h"

int parse_args(char *argv[], char *buf);	/* see parse.c */

double	atof(), fabs();

void close_notify_fn(port)
int port;
    {
    fprintf(stderr, "Port number %d closed; exiting...\n", port);
    exit(0);
    }

void help()
    {
    printf("\nQScript Commands:\n\n");
    printf("  GoTo cell_id                 MoveTo cell_id\n");
    printf("  Put Number x                 Put Label text\n");
    printf("  Put Formula text             Put Message text\n");
    printf("\n");
    printf("  Recalc                       Delay seconds\n");
    printf("\n");
    printf("  Copy Formulas src dst        Insert Rows range\n");
    printf("  Copy Values src dst          Insert Cols range\n");
    printf("  Move src dst                 Delete Rows range\n");
    printf("  Erase range                  Delete Cols range\n");
    printf("\n");
    printf("  Write filename               ClearAll\n");
    printf("\n");
    printf("  SetFormat *|e|f|g|$|,|h|l|a|b|c|i|t places range\n");
    printf("  SetFont default|normal|bold|italic|bold-italic range\n");
    printf("\n");
    printf("  Protect range                Unprotect range\n");
    printf("\n");
    printf("  SetTitleRows loc             ClearTitleRows\n");
    printf("  SetTitleCols loc             ClearTitleCols\n");
    printf("  SetTitleBoth loc             ClearTitleBoth\n\n");
    printf("\n");
    printf("  SelectGraph [0-9]            SelectDataset [1-5]\n");
    printf("  GraphType [0-7]              DataRange x|y|z [range]\n");
    printf("  Shade [0-2]                  LegendText [text]\n");
    printf("  Update [0|1]                 MarkerType [0-5]\n");
    printf("  Legend [0|1]                 LineStyle [1-5]\n");
    printf("  TitleFont [1-4]              SegmentType [0-2]\n");
    printf("  GraphTitle [text]            Disable [0|1]\n");
    printf("\n");
    printf("  LabelText x|y|z [text]       Rotate [theta phi]\n");
    printf("  LabelFont x|y|z [1-4]\n");
    printf("  AutoTicks x|y|z              Redraw\n");
    printf("  SetTicks x|y|z [n]           Name [name]\n");
    printf("  AutoScale x|y|z              Duplicate\n");
    printf("  SetScale x|y|z [min max]     Delete\n");
    printf("  OutTicks x|y|z [0|1]         Print eps|idraw [filename]\n");
    printf("  Exploded x|y|z [0|1]         Cancel\n");
    printf("  GridLines x|y|z [0|1]\n");
    printf("                               NewGraph\n");
    printf("  Geometry [x y width height]  DisplayGraph\n");
    printf("\n");
    }

int get_axis(char *s)
    {
    if (strlen(s) != 1) return -1;
    switch (*s)
	{
	case 'x': case 'X':	return 0;
	case 'y': case 'Y':	return 1;
	case 'z': case 'Z':	return 2;
	default:		return -1;
	}
    }

void cvt2lower(char *buf)
    {
    int c;
    while ((c = *buf))
	{
	if (isupper(c)) *buf = tolower(c);
	++buf;
	}
    }

int cmdcmp(char *s, char *cmd)
    {
    cvt2lower(s);
    return !strcmp(s, cmd);
    }

int main(argc, argv)
int argc;
char *argv[];
    {
    char buf[512], name_buf[512];
    int port, axis, row, col, fc;
    int gid = 0, dataset = 1, status, new_gid, print_type;
    Range src, dst;
    unsigned long mask;
    GrGenChar genchar;
    GrAxisChar axischar;
    XS2GrDataChar dchar;
    GrRotate3D grotate;
    GrGeometry ggeom;

    help();

    nexs_set_close_notify(close_notify_fn);
    port = nexs_open_connection("QScript", 60);
    if (port == 0)
	{
	fprintf(stderr, "could not connect to NExS after 60 seconds...\n");
	exit(-1);
	}

    while (1)
	{
	mask = 0;
	nexs_flush_port(port);
	/*printf("Command> ");*/
	if (!fgets(buf, sizeof(buf), stdin)) exit(0);
	argc = parse_args(argv, buf);
	if (argc < 1) continue;
	if (cmdcmp(argv[0], "quit"))
	    {
	    exit(0);
	    }
	else if (cmdcmp(argv[0], "help"))
	    {
	    help();
	    }
	
	else if (cmdcmp(argv[0], "moveto"))
	    {
	    if (argc != 2) continue;
	    status = text_to_rc(argv[1], &row, &col);
	    if (status) nexs_moveto(port, row, col);
	    }
	else if (cmdcmp(argv[0], "goto"))
	    {
	    if (argc != 2) continue;
	    status = text_to_rc(argv[1], &row, &col);
	    if (status) nexs_goto(port, row, col);
	    }
	else if (cmdcmp(argv[0], "put"))
	    {
	    if (argc != 3) continue;
	    status = nexs_get_location(port, &row, &col);
	    if (status)
		{
		if (cmdcmp(argv[1], "number"))
		    {
		    nexs_store_number(port, row, col, atof(argv[2]));
		    }
		if (cmdcmp(argv[1], "label"))
		    {
		    nexs_store_label(port, row, col, argv[2]);
		    }
		if (cmdcmp(argv[1], "formula"))
		    {
		    nexs_store_formula(port, row, col, argv[2]);
		    }
		if (cmdcmp(argv[1], "message"))
		    {
		    nexs_display_message(port, argv[2]);
		    }
		}
	    }
	else if (cmdcmp(argv[0], "recalc"))
	    {
	    nexs_recalc(port, 1, 0, NULL);
	    }
	else if (cmdcmp(argv[0], "delay"))
	    {
	    if (argc != 2) continue;
	    sleep(atoi(argv[1]));
	    }

/*
 * Start of Editing Commands
 */

	else if (cmdcmp(argv[0], "copy"))
	    {
	    if (argc != 4) continue;
	    if (cmdcmp(argv[1], "formulas"))
		{
		if (!text_to_range(argv[2], &src)) continue;
		if (!text_to_range(argv[3], &dst)) continue;
		nexs_copy_range(port, 0, src, dst);
		}
	    else if (cmdcmp(argv[1], "values"))
		{
		if (!text_to_range(argv[3], &src)) continue;
		if (!text_to_range(argv[3], &dst)) continue;
		nexs_copy_range(port, 1, src, dst);
		}
	    }
	else if (cmdcmp(argv[0], "move"))
	    {
	    if (argc != 3) continue;
	    if (!text_to_range(argv[1], &src)) continue;
	    if (!text_to_range(argv[2], &dst)) continue;
	    nexs_move_range(port, src, dst.r0, dst.c0);
	    }
	else if (cmdcmp(argv[0], "erase"))
	    {
	    if (argc != 2) continue;
	    if (!text_to_range(argv[1], &dst)) continue;
	    nexs_erase_range(port, dst);
	    }
	else if (cmdcmp(argv[0], "setformat"))
	    {
	    if (argc != 4) continue;
	    if (!text_to_range(argv[3], &dst)) continue;
	    nexs_set_format(port, *argv[1], atoi(argv[2]), dst);
	    }
	else if (cmdcmp(argv[0], "setfont"))
	    {
	    if (argc != 3) continue;
	    if (cmdcmp(argv[1], "default"))		fc = FNT_DEF;
	    else if (cmdcmp(argv[1], "normal"))	fc = FNT_NORMAL;
	    else if (cmdcmp(argv[1], "bold"))		fc = FNT_BOLD;
	    else if (cmdcmp(argv[1], "italic"))	fc = FNT_ITALIC;
	    else if (cmdcmp(argv[1], "bold-italic"))	fc = FNT_BOLD_ITALIC;
	    else continue;
	    if (!text_to_range(argv[2], &dst)) continue;
	    nexs_set_font(port, fc, dst);
	    }
	else if (cmdcmp(argv[0], "insert"))
	    {
	    if (argc != 3) continue;
	    if (cmdcmp(argv[1], "rows"))
		{
		if (!text_to_range(argv[2], &dst)) continue;
		nexs_insert_rows(port, dst.r0, dst.r1 - dst.r0 + 1);
		}
	    else if (cmdcmp(argv[1], "cols"))
		{
		if (!text_to_range(argv[2], &dst)) continue;
		nexs_insert_cols(port, dst.c0, dst.c1 - dst.c0 + 1);
		}
	    }
	else if (cmdcmp(argv[0], "delete"))
	    {
	    if (argc != 3) continue;
	    else if (cmdcmp(argv[1], "row") ||
		     cmdcmp(argv[1], "rows"))
		{
		if (!text_to_range(argv[2], &dst)) continue;
		nexs_delete_rows(port, dst.r0, dst.r1);
		}
	    else if (cmdcmp(argv[1], "col") ||
		     cmdcmp(argv[1], "cols"))
		{
		if (!text_to_range(argv[2], &dst)) continue;
		nexs_delete_cols(port, dst.c0, dst.c1);
		}
	    }
	else if (cmdcmp(argv[0], "clearall"))
	    {
	    nexs_clear_sheet(port);
	    }
	else if (cmdcmp(argv[0], "protect"))
	    {
	    if (argc != 2) continue;
	    if (!text_to_range(argv[1], &dst)) continue;
	    nexs_protect(port, 1, dst);
	    }
	else if (cmdcmp(argv[0], "unprotect"))
	    {
	    if (argc != 2) continue;
	    if (!text_to_range(argv[1], &dst)) continue;
	    nexs_protect(port, 0, dst);
	    }
	else if (cmdcmp(argv[0], "settitlerows"))
	    {
	    if (argc != 2) continue;
	    if (!text_to_range(argv[1], &dst)) continue;
	    nexs_set_titles(port, dst.r0, dst.c0, TITLE_ROWS);
	    }
	else if (cmdcmp(argv[0], "settitlecols"))
	    {
	    if (argc != 2) continue;
	    if (!text_to_range(argv[1], &dst)) continue;
	    nexs_set_titles(port, dst.r0, dst.c0, TITLE_COLS);
	    }
	else if (cmdcmp(argv[0], "settitleboth"))
	    {
	    if (argc != 2) continue;
	    if (!text_to_range(argv[1], &dst)) continue;
	    nexs_set_titles(port, dst.r0, dst.c0, TITLE_BOTH);
	    }
	else if (cmdcmp(argv[0], "cleartitlerows"))
	    {
	    if (argc != 1) continue;
	    nexs_clear_titles(port, TITLE_ROWS);
	    }
	else if (cmdcmp(argv[0], "cleartitlecols"))
	    {
	    if (argc != 1) continue;
	    nexs_clear_titles(port, TITLE_COLS);
	    }
	else if (cmdcmp(argv[0], "cleartitleboth"))
	    {
	    if (argc != 1) continue;
	    nexs_clear_titles(port, TITLE_BOTH);
	    }

/*
 * Start of Graphics Commands
 */

	else if (cmdcmp(argv[0], "selectgraph"))
	    {
	    if (argc > 1)
		{
		gid = atoi(argv[1]);
		}
	    printf(" = %d\n", gid);
	    }
	else if (cmdcmp(argv[0], "graphtype"))
	    {
	    if (argc > 1)
		{
		mask = GR_TYPE;
		genchar.graph_type = atoi(argv[1]);
		}
	    if (!nexs_ggen_char(port, gid, &genchar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", genchar.graph_type);
	    free(genchar.title_text);
	    }
	else if (cmdcmp(argv[0], "shade"))
	    {
	    if (argc > 1)
		{
		mask = SURFACE_SHADE;
		genchar.surface_shade = atoi(argv[1]);
		}
	    if (!nexs_ggen_char(port, gid, &genchar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", genchar.surface_shade);
	    free(genchar.title_text);
	    }
	else if (cmdcmp(argv[0], "update"))
	    {
	    if (argc > 1)
		{
		mask = RECALC_UPDATE;
		genchar.recalc_update = atoi(argv[1]);
		}
	    if (!nexs_ggen_char(port, gid, &genchar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", genchar.recalc_update);
	    free(genchar.title_text);
	    }
	else if (cmdcmp(argv[0], "legend"))
	    {
	    if (argc > 1)
		{
		mask = DISP_LEGEND;
		genchar.disp_legend = atoi(argv[1]);
		}
	    if (!nexs_ggen_char(port, gid, &genchar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", genchar.disp_legend);
	    free(genchar.title_text);
	    }
	else if (cmdcmp(argv[0], "titlefont"))
	    {
	    if (argc > 1)
		{
		mask = TITLE_FONT;
		genchar.title_font = atoi(argv[1]);
		}
	    if (!nexs_ggen_char(port, gid, &genchar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", genchar.title_font);
	    free(genchar.title_text);
	    }
	else if (cmdcmp(argv[0], "graphtitle"))
	    {
	    if (argc > 1)
		{
		mask = TITLE_TEXT;
		genchar.title_text = argv[1];
		}
	    if (!nexs_ggen_char(port, gid, &genchar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %s\n", genchar.title_text);
	    free(genchar.title_text);
	    }
	else if (cmdcmp(argv[0], "axislabel"))
	    {
	    if (argc < 2) continue;
	    axis = get_axis(argv[1]);
	    if (argc > 2)
		{
		mask = LABEL_TEXT;
		axischar.label_text = argv[2];
		}
	    if (!nexs_axis_char(port, gid, axis, &axischar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %s\n", axischar.label_text);
	    free(axischar.label_text);
	    }
	else if (cmdcmp(argv[0], "labelfont"))
	    {
	    if (argc < 2) continue;
	    axis = get_axis(argv[1]);
	    if (argc > 2)
		{
		mask = LABEL_FONT;
		axischar.label_font = atoi(argv[2]);
		}
	    if (!nexs_axis_char(port, gid, axis, &axischar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", axischar.label_font);
	    free(axischar.label_text);
	    }
	else if (cmdcmp(argv[0], "numticks"))
	    {
	    if (argc < 2) continue;
	    axis = get_axis(argv[1]);
	    if (argc > 2)
		{
		mask = LABEL_FONT;
		axischar.num_ticks = atoi(argv[2]);
		}
	    if (!nexs_axis_char(port, gid, axis, &axischar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", axischar.num_ticks);
	    free(axischar.label_text);
	    }
	else if (cmdcmp(argv[0], "logscale"))
	    {
	    if (argc < 2) continue;
	    axis = get_axis(argv[1]);
	    if (argc > 2)
		{
		mask = LOG_SCALE;
		axischar.log_scale = atoi(argv[2]);
		}
	    if (!nexs_axis_char(port, gid, axis, &axischar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", axischar.log_scale);
	    free(axischar.label_text);
	    }
	else if (cmdcmp(argv[0], "autoticks"))
	    {
	    if (argc < 2) continue;
	    axis = get_axis(argv[1]);
	    mask = AUTO_TICKS;
	    axischar.auto_ticks = 1;
	    if (!nexs_axis_char(port, gid, axis, &axischar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", axischar.auto_ticks);
	    free(axischar.label_text);
	    }
	else if (cmdcmp(argv[0], "setticks"))
	    {
	    if (argc < 2) continue;
	    axis = get_axis(argv[1]);
	    if (argc > 2)
		{
		mask = AUTO_TICKS | NUM_TICKS;
		axischar.auto_ticks = 0;
		axischar.num_ticks = atoi(argv[2]);
		}
	    if (!nexs_axis_char(port, gid, axis, &axischar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", axischar.num_ticks);
	    free(axischar.label_text);
	    }
	else if (cmdcmp(argv[0], "autoscale"))
	    {
	    if (argc < 2) continue;
	    axis = get_axis(argv[1]);
	    mask = AUTO_SCALE;
	    axischar.auto_scale = 1;
	    if (!nexs_axis_char(port, gid, axis, &axischar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", axischar.auto_scale);
	    free(axischar.label_text);
	    }
	else if (cmdcmp(argv[0], "setscale"))
	    {
	    if (argc < 2) continue;
	    axis = get_axis(argv[1]);
	    if (argc > 3)
		{
		mask = AUTO_SCALE | AXIS_MIN | AXIS_MAX;
		axischar.auto_scale = 0;
		axischar.axis_min = atof(argv[2]);
		axischar.axis_max = atof(argv[3]);
		}
	    if (!nexs_axis_char(port, gid, axis, &axischar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %g, %g\n", axischar.axis_min, axischar.axis_max);
	    free(axischar.label_text);
	    }
	else if (cmdcmp(argv[0], "outticks"))
	    {
	    if (argc < 2) continue;
	    axis = get_axis(argv[1]);
	    if (argc > 2)
		{
		mask = OUT_TICKS;
		axischar.out_ticks = atoi(argv[2]);
		}
	    if (!nexs_axis_char(port, gid, axis, &axischar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", axischar.out_ticks);
	    free(axischar.label_text);
	    }
	else if (cmdcmp(argv[0], "exploded"))
	    {
	    if (argc < 2) continue;
	    axis = get_axis(argv[1]);
	    if (argc > 2)
		{
		mask = EXPLODED_AXIS;
		axischar.exploded_axis = atoi(argv[2]);
		}
	    if (!nexs_axis_char(port, gid, axis, &axischar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", axischar.exploded_axis);
	    free(axischar.label_text);
	    }
	else if (cmdcmp(argv[0], "gridlines"))
	    {
	    if (argc < 2) continue;
	    axis = get_axis(argv[1]);
	    if (argc > 2)
		{
		mask = GRID_LINES;
		axischar.grid_lines = atoi(argv[2]);
		}
	    if (!nexs_axis_char(port, gid, axis, &axischar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", axischar.grid_lines);
	    free(axischar.label_text);
	    }
	else if (cmdcmp(argv[0], "selectdataset"))
	    {
	    if (argc > 1)
		{
		dataset = atoi(argv[1]);
		}
	    printf(" = %d\n", dataset);
	    }
	else if (cmdcmp(argv[0], "datarange"))
	    {
	    if (argc < 2) continue;
	    axis = get_axis(argv[1]);
	    if (argc > 2)
		{
		switch (axis)
		    {
		    case 0:	mask = X_RANGE;
				dchar.x_range = argv[2];
				break;
		    case 1:	mask = Y_RANGE;
				dchar.y_range = argv[2];
				break;
		    case 2:	mask = Z_RANGE;
				dchar.z_range = argv[2];
				break;
		    }
		}
	    if (!xs2_gdata_char(port, gid, dataset, &dchar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    switch (axis)
		{
		case 0:	printf(" = %s\n", dchar.x_range);
			break;
		case 1:	printf(" = %s\n", dchar.y_range);
			break;
		case 2:	printf(" = %s\n", dchar.z_range);
			break;
		}
	    free(dchar.x_range);
	    free(dchar.y_range);
	    free(dchar.z_range);
	    free(dchar.legend_text);
	    }
	else if (cmdcmp(argv[0], "legendtext"))
	    {
	    if (argc > 1)
		{
		dchar.legend_text = argv[1];
		mask = LEGEND_TEXT;
		}
	    if (!xs2_gdata_char(port, gid, dataset, &dchar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %s\n", dchar.legend_text);
	    free(dchar.x_range);
	    free(dchar.y_range);
	    free(dchar.z_range);
	    free(dchar.legend_text);
	    }
	else if (cmdcmp(argv[0], "markertype"))
	    {
	    if (argc > 1)
		{
		dchar.marker_type = atoi(argv[1]);
		mask = MARKER_TYPE;
		}
	    if (!xs2_gdata_char(port, gid, dataset, &dchar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", dchar.marker_type);
	    free(dchar.x_range);
	    free(dchar.y_range);
	    free(dchar.z_range);
	    free(dchar.legend_text);
	    }
	else if (cmdcmp(argv[0], "linestyle"))
	    {
	    if (argc > 1)
		{
		dchar.line_style = atoi(argv[1]);
		mask = LINE_STYLE;
		}
	    if (!xs2_gdata_char(port, gid, dataset, &dchar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", dchar.line_style);
	    free(dchar.x_range);
	    free(dchar.y_range);
	    free(dchar.z_range);
	    free(dchar.legend_text);
	    }
	else if (cmdcmp(argv[0], "segmenttype"))
	    {
	    if (argc > 1)
		{
		dchar.segment_type = atoi(argv[1]);
		mask = SEGMENT_TYPE;
		}
	    if (!xs2_gdata_char(port, gid, dataset, &dchar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", dchar.segment_type);
	    free(dchar.x_range);
	    free(dchar.y_range);
	    free(dchar.z_range);
	    free(dchar.legend_text);
	    }
	else if (cmdcmp(argv[0], "disable"))
	    {
	    if (argc > 1)
		{
		dchar.data_disable = atoi(argv[1]);
		mask = DATA_DISABLE;
		}
	    if (!xs2_gdata_char(port, gid, dataset, &dchar, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", dchar.data_disable);
	    free(dchar.x_range);
	    free(dchar.y_range);
	    free(dchar.z_range);
	    free(dchar.legend_text);
	    }
	else if (cmdcmp(argv[0], "rotate"))
	    {
	    if (argc > 2)
		{
		grotate.theta = atof(argv[1]);
		grotate.phi = atof(argv[2]);
		mask = THETA | PHI;
		}
	    if (!nexs_rotate_graph(port, gid, &grotate, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %g, %g\n", grotate.theta, grotate.phi);
	    }
	else if (cmdcmp(argv[0], "geometry"))
	    {
	    if (argc > 4)
		{
		ggeom.x = atoi(argv[1]);
		ggeom.y = atoi(argv[2]);
		ggeom.width = atoi(argv[3]);
		ggeom.height = atoi(argv[4]);
		mask = GR_X_POS | GR_Y_POS | GR_WIDTH | GR_HEIGHT;
		}
	    if (!nexs_graph_geometry(port, gid, &ggeom, mask))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d, %d, %d, %d\n",
			ggeom.x, ggeom.y, ggeom.width, ggeom.height);
	    }
	else if (cmdcmp(argv[0], "redraw"))
	    {
	    status = nexs_redraw_graph(port, gid);
	    printf("status = %d\n", status);
	    }
	else if (cmdcmp(argv[0], "name"))
	    {
	    if (argc > 1)
		strcpy(name_buf, argv[1]);	/* set name */
	    else
		*name_buf = '\0';		/* read name */
	    if (!nexs_name_graph(port, gid, name_buf))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %s\n", name_buf);
	    }
	else if (cmdcmp(argv[0], "duplicate"))
	    {
	    if (!nexs_duplicate_graph(port, gid, &new_gid))
		{
		printf(" = ???\n");
		continue;
		}
	    printf(" = %d\n", new_gid);
	    }
	else if (cmdcmp(argv[0], "delete"))
	    {
	    status = nexs_delete_graph(port, gid);
	    printf("status = %d\n", status);
	    }
	else if (cmdcmp(argv[0], "print"))
	    {
	    if (argc < 2) continue;
	    if (cmdcmp(argv[1], "eps"))
		print_type = GR_EPS;
	    else if (cmdcmp(argv[1], "idraw"))
		print_type = GR_IDRAW_PS;
	    else
		continue;
	    if (argc > 2)
		strcpy(name_buf, argv[2]);	/* print file name */
	    else
		*name_buf = '\0';		/* send to printer */
	    status = nexs_print_graph(port, gid, print_type, name_buf);
	    printf("status = %d\n", status);
	    }
	else if (cmdcmp(argv[0], "cancel"))
	    {
	    status = nexs_cancel_graph(port, gid);
	    printf("status = %d\n", status);
	    }
	else if (cmdcmp(argv[0], "newgraph"))
	    {
	    status = nexs_new_graph(port, &new_gid);
	    printf(" = %d\n", new_gid);
	    }
	else if (cmdcmp(argv[0], "displaygraph"))
	    {
	    status = nexs_display_graph(port, gid);
	    printf("status = %d\n", status);
	    }
	else if (cmdcmp(argv[0], "write"))
	    {
	    status = nexs_write_file(port, argv[1]);
	    printf("status = %d\n", status);
	    }

	else help();
	}
    }
