/*
 *
 *  NExS Connection Library V2.0
 *  Copyright 1990, 1991, X Engineering Software Systems Corporation
 *  Copyright 1999, GreyTrout Software, Inc.
 *  All rights reserved
 *
 *  file: server.c
 *
 * $Id: server.c,v 1.2 91/08/14 17:23:55 tkm Exp $
 *
 */


#include	<stdio.h>
#include	<stdlib.h>
#include	<signal.h>
#include	<unistd.h>
#include	<X11/Xlib.h>
#include	<X11/Xatom.h>
#include	"nexs.h"

Atom	xsConnectAtom;
Display	*display;
Window	nexs_window();

double number = 0.0;
char *strptr = NULL;
int status;

Operand fn_demo1(argv)
Operand argv[];
/*
  Function DEMO1(x).  Returns 2 times its argument, if the argument is a
  number or a cell.  Returns 0 otherwise.
*/
    {
    Operand op;
    double x;

    switch (argv[1].type)
	{
	case ARG_NUMBER:
	    op.val.number = argv[1].val.number * 2.0;
	    break;
	case ARG_CELL:
	    nexs_get_number(nexs_port,
		argv[1].val.range.r0, argv[1].val.range.c0, &x);
	    op.val.number = 2.0 * x;
	    break;
	default:
	    op.val.number = 0.0;
	}
    op.type = ARG_NUMBER;
    return op;
    }

Operand fn_demo2(argv)
Operand argv[];
/*
  Function DEMO2(x).  Returns 0.5 times its argument, if the argument is a
  number or a cell.  Returns 0 otherwise.
*/
    {
    Operand op;
    double x;

    switch (argv[1].type)
	{
	case ARG_NUMBER:
	    op.val.number = argv[1].val.number * 0.5;
	    break;
	case ARG_CELL:
	    nexs_get_number(nexs_port,
		argv[1].val.range.r0, argv[1].val.range.c0, &x);
	    op.val.number = 0.5 * x;
	    break;
	default:
	    op.val.number = 0.0;
	}
    op.type = ARG_NUMBER;
    return op;
    }

void copy_button()
    {
    int r, c;

    nexs_get_location(nexs_port, &r, &c);
    if (strptr)
	{
	free(strptr);
	strptr = NULL;
	}
    status = nexs_get_value(nexs_port, r, c, &number, &strptr); 
    }

void paste_button()
    {
    int r, c;

    nexs_get_location(nexs_port, &r, &c);
    switch (status)
	{
	case NEXS_NUMBER:
	    nexs_store_number(nexs_port, r, c, number);
	    break;
	case NEXS_STRING:
	    nexs_store_label(nexs_port, r, c, strptr);
	    break;
	}
    }

void check_for_nexs()
/*
  Checks for a new nexs "client".  A new client is identified by the
  fact that it owns the xsConnectAtom and its window ID is different
  from any of the currently active ports.
*/
    {
    Window w;
    int portID, i;
    long menu;

    if ((w = XGetSelectionOwner(display, xsConnectAtom)) != None)
	{
	for (i=1; i<=MAX_PORTS; i++)
	    if (w == nexs_window(i)) return;
	portID = nexs_establish_connection("ServerDemo", 0);
	if (portID)
	    {
	    nexs_install_function(portID, "DEMO1", fn_demo1, 0);
	    nexs_install_function(portID, "DEMO2", fn_demo2, 0);
	    menu = nexs_create_menu(portID, "Server");
	    nexs_create_button(portID, menu, "Copy", copy_button, 0);
	    nexs_create_button(portID, menu, "Paste", paste_button, 0);
	    }
	}
    }

void send_time2check()
/*
  Send a ClientMessage event to this process once every second which
  says its time to look and see if any new copies of nexs have come
  up which need to be served.
*/
   {
   XClientMessageEvent cm_event;

   cm_event.type = ClientMessage;
   cm_event.display = display;
   cm_event.window = nexs_window(0);	/* returns client window */
   cm_event.message_type = xsConnectAtom;	/* irrelevant */
   cm_event.format = 32;
   XSendEvent(display, cm_event.window, False, None, (XEvent *) &cm_event);
   XFlush(display);
   signal(SIGALRM, send_time2check);	/* set up to check again */
   alarm(1);				/* in one second */
   }

void check_event(event)
XEvent *event;
   {
   if (event->type == ClientMessage &&
       event->xclient.message_type == xsConnectAtom)
	check_for_nexs();
   }

int main(argc, argv)
int argc;
char *argv[];
    {
    if ((display = XOpenDisplay("")) == NULL)
	{
	fprintf(stderr, "Could not open display!\n");
	exit(0);
	}
    nexs_connect_init(display);
    xsConnectAtom = XInternAtom(display, "XS_Connect", False);
    send_time2check();
    nexs_dispatcher(check_event);
	 return 0;
    }
