/*
 *
 *  NExS Connection Library V2.0
 *  Copyright 1990, 1991, X Engineering Software Systems Corporation
 *  Copyright 1999, GreyTrout Software, Inc.
 *  All rights reserved
 *
 *  file: test15.c
 *
 * $Id: test15.c,v 1.2 91/08/13 18:49:41 tkm Exp $
 *
 */


#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>
#include	"nexs.h"

double	atof(), fabs();

void close_notify_fn(port)
int port;
    {
    fprintf(stderr, "Port number %d closed; exiting...\n", port);
    exit(0);
    }

int parse_args(argv, buf)
char *argv[], *buf;
    {
    int argc = 0, history = 0;
    while (*buf)
	{
	if (isspace(*buf))
	    {
	    *buf++ = '\0';
	    history = 0;
	    }
	else if (!history)
	    {
	    argv[argc++] = buf++;
	    history = 1;
	    }
	else
	    buf++;
	}
    return argc;
    }

void help()
    {
    printf("\nCommands:\n");
    printf("  Sort asc|desc update|noupdate range key1 [key2]\n");
    printf("  GoalSeek variable_cell target_cell value [b1 b2] [maxit] [tolerance]\n");
    printf("  Find next|prev bycol|byrow range condition\n");
    printf("  Extract src dst keycol condition\n");
    printf("\n");
    }

int main(argc, argv)
int argc;
char *argv[];
    {
    char buf[100];
    int port;
    int dir, mode, keycol1, keycol2, vr, vc, tr, tc, maxit, err,
        order, type, find_r, find_c, r_dst, c_dst, status;
    Range src, dst, range;
    double target, b1, b2, tolerance;

    help();

    nexs_set_close_notify(close_notify_fn);
    port = nexs_open_connection("Test 15", 10);
    if (port == 0)
	{
	printf("could not connect to NExS after 10 seconds...\n");
	exit(0);
	}

    while (1)
	{
	printf("Command> ");
	if (!fgets(buf, sizeof(buf), stdin)) exit(0);
	argc = parse_args(argv, buf);
	if (argc < 1) continue;
	if (!strcmp(argv[0], "quit"))
	    {
	    exit(0);
	    }
	else if (!strcmp(argv[0], "help"))
	    {
	    help();
	    }

	else if (!strcmp(argv[0], "Sort"))
	    {
	    if (argc < 5) continue;
	    if (!text_to_range(argv[3], &dst)) continue;
	    if (!text_to_range(argv[4], &src)) continue;
	    keycol2 = keycol1 = src.c0;
	    if (argc > 5)
		{
		if (!text_to_range(argv[5], &src)) continue;
		keycol2 = src.c0;
		}
	    if (!strcmp(argv[1], "asc"))	dir = SORT_ASCENDING;
	    else if (!strcmp(argv[1], "desc"))	dir = SORT_DESCENDING;
	    else continue;
	    if (!strcmp(argv[2], "update"))		mode = SORT_UPDATE;
	    else if (!strcmp(argv[2], "noupdate"))	mode = SORT_NO_UPDATE;
	    else continue;
	    nexs_sort(port, dir, mode, dst, keycol1, keycol2);
	    }
	else if (!strcmp(argv[0], "GoalSeek"))
	    {
	    if (argc < 4) continue;
	    if (!text_to_rc(argv[1], &vr, &vc)) continue;
	    if (!text_to_rc(argv[2], &tr, &tc)) continue;
	    target = atof(argv[3]);
	    if (argc > 5)
		{
		b1 = atof(argv[4]);
		b2 = atof(argv[5]);
		}
	    else	/* take variable cell contents as initial guess */
		{
		if (!nexs_get_number(port, vr, vc, &b1))
		    {
		    printf("Variable cell does not contain numeric constant\n");
		    continue;
		    }
		b2 = b1 + 1.0;
		b1 -= 1.0;
		}
	    if (argc > 6)
		maxit = atoi(argv[6]);
	    else
		maxit = 10;
	    if (argc > 7)
		tolerance = atof(argv[7]);
	    else
		tolerance = fabs(target) * 1e-12;
	    err = nexs_goalseek(port, vr, vc, tr, tc, target,
						&b1, &b2, maxit, tolerance);
	    if (err > 0)
		printf("Target reached in %d iterations\n", err);
	    else
	      switch (err)
		{
		case GS_FORMULA_ERR:
		    printf("Target cell does not contain numeric formula\n");
		    break;
		case GS_VARIABLE_ERR:
		    printf("Variable cell does not contain numeric constant\n");
		    break;
		case GS_FP_ERR:
		    printf("Floating point error while goalseeking\n");
		    break;
		case GS_TARGET_ERR:
		    printf("Non-numeric value occurred in target cell\n");
		    break;
		case GS_BRACKET_ERR:
		    printf("Couldn't bracket target after %d iterations\n",
									maxit);
		    break;
		case GS_ITER_ERR:
		    printf("Couldn't converge to target after %d iterations\n",
									maxit);
		    break;
		}
	    }
	else if (!strcmp(argv[0], "Find"))
	    {
	    if (argc < 5) continue;
	    if (!strcmp(argv[1], "next"))	dir = FIND_NEXT;
	    else if (!strcmp(argv[1], "prev"))	dir = FIND_PREV;
	    else continue;
	    if (!strcmp(argv[2], "bycol"))	order = FIND_BY_COL;
	    else if (!strcmp(argv[2], "byrow"))	order = FIND_BY_ROW;
	    else continue;
	    if (!text_to_range(argv[3], &range)) continue;
	    nexs_get_location(port, &find_r, &find_c);
	    if (find_r < range.r0 || find_r > range.r1 ||
		find_c < range.c0 || find_c > range.c1)
		{
		find_r = range.r0;
		find_c = range.c0;
		}
	    if (index(argv[4], '#'))
		type = FIND_NUMERIC;
	    else
		type = FIND_REGEX;
	    status = nexs_find(port, order, dir, range, argv[4], type,
							&find_r, &find_c);
	    if (status)
		{
		nexs_moveto(port, find_r, find_c);
		nexs_flush();
		}
	    else
		printf("Target not found\n");
	    }
	else if (!strcmp(argv[0], "Extract"))
	    {
	    if (argc < 5) continue;
	    if (!text_to_range(argv[1], &src)) continue;
	    if (!text_to_rc(argv[2], &r_dst, &c_dst)) continue;
	    if ((keycol1 = text_to_col(argv[3])) == -1) continue;
	    if (index(argv[4], '#'))
		type = FIND_NUMERIC;
	    else
		type = FIND_REGEX;
	    status = nexs_extract(port, src, keycol1, argv[4], type,
								r_dst, c_dst);
	    if (status < 0)
		printf("Error in extract condition\n");
	    else
		printf("%d record(s) extracted\n", status);
	    }
	    
	else help();
	}
	 return 0;
    }
