/*
 *
 * NExS Connection Library V1.0 (Beta release B, September 10, 1990)
 * Copyright 1990, X Engineering Software Systems Corporation
 *  Copyright 1999, GreyTrout Software, Inc.
 * All rights reserved.
 *
 * The author hereby grants permission to use, modify, and distribute
 * this program so long as the above copyright message is retained.
 *
 * file: test5.c
 *
 * (This example program has been edited for compactness and clarity.
 *  - TKM 10/15/95)
 *
 */


#include        <stdio.h>
#include        <nexs.h>

int portID;

static int error_flag;  /* set if any function arg is a string */

static Number sum_args(Operand *argv)   /* compute sum of function args */
    {
    Number sum, x;
    int i, n = argv->val.count;
    int r, c;

    sum = 0.;
    error_flag = 0;
    for (i=0; i<n; i++)
        {
        switch ((++argv)->type)
            {
            case ARG_NUMBER:    /* add the number to the sum */
                sum += argv->val.number;
                break;
            case ARG_CELL:      /* add the cell contents to the sum */
                if (nexs_get_number(portID,
                        argv->val.range.r0, argv->val.range.c0, &x))
                    sum += x;
                break;
            case ARG_RANGE:     /* sum the cell values in the range */
                for (r = argv->val.range.r0; r <= argv->val.range.r1; r++)
                  for (c = argv->val.range.c0; c <= argv->val.range.c1; c++)
                        if (nexs_get_number(portID, r, c, &x))
                            sum += x;
                break;
            case ARG_STRING:    /* make it an error message */
                strcpy(nexs_sbuf, argv->val.string);
                error_flag = 1;
                return 0.0;
            default:
                break;  /* ignore it */
            }
        }

    return sum;
    }

Operand nexs_fn(Operand *argv)  /* identical to @sum(...) */
    {
    Operand op;

    op.val.number = sum_args(argv);
    if (error_flag)
        op.type = ARG_ERROR;
    else
        op.type = ARG_NUMBER;
    return op;
    }

Operand nexs2_fn(Operand *argv) /* returns a string */
    {
    Operand op;

    if (error_flag)
        op.type = ARG_ERROR;
    else
        {
        op.type = ARG_STRING;
        sprintf(nexs_sbuf, "The sum is %g", sum_args(argv));
        }
    return op;
    }

Operand nexs3_fn(Operand *argv)  /* stores a "tagged number" (embedded tool) */
    {
    Operand op;

    if (error_flag)
        op.type = ARG_ERROR;
    else
        {
        op.val.number = sum_args(argv);
        nexs_store_tagged_number(portID, nexs_row+1, nexs_col, op.val.number*2);
        op.type = ARG_NUMBER;
        }
    return op;
    }

void close_notify_fn(int port)
    {
    fprintf(stderr, "Port number %d closed; exiting...\n", port);
    exit(0);
    }

int main(argc, argv)
int argc;
char *argv[];
	{
    nexs_set_close_notify(close_notify_fn);
    portID = nexs_open_connection("Functions", 10);
    if (portID == 0)
        {
        printf("could not connect to NExS after 10 seconds...\n");
        exit(0);
        }
    nexs_install_function(portID, "NEXS", nexs_fn, 0);
    nexs_install_function(portID, "NEXS2", nexs2_fn, 0);
    nexs_install_function(portID, "NEXS3", nexs3_fn, 0);
    nexs_dispatcher(NULL);

    return 0;
    }
