static char rcsid[] = "$Id: nexsAxisChar.c,v 1.1 1998/03/23 02:58:36 tkm Exp $";

/*------------------------------------------------------------*
|                                                             |
| Interface for NExS  gtaph API     .                         |
| Author:	Damir Delija                                  |
|               University Computing Center, SRCE             |
| Date:         Wed Mar 12 1997                               |
|               Copyright (c) 1997,  Damir Delija             |
|                                                             |
*--------------------Procedure Description--------------------*
|                                                             |
| nexs_axis_char.c D. Delija 12/03/97                         |
|                                                             |
| int nexs_axis_char(int port, int gid, int axis, GrAxisChar  |
| axischar, int mask);                                        |
|                                                             |
|       Gets and/or sets all fields on the "X|Y|Z Axis        |
|       Characteristics" dialog of the graph specified by     |
|       "gid". The axis to be manipulated is specified        |
|       by the parameter "axis", which must be set to one     |
|       the following valued defined in "nexs.h": X_AXIS,     |
|       Y_AXIS, Z_AXIS, AUX_Y_AXIS.  To set fields on the     |
|       graph's X|Y|Z Axis Characteristics dialog, the        |
|       desired values should be stored in the "axischar"     |
|       structure and the corresponding bits in "mask"        |
|       should be set to 1.  See "nexs.h" for details of      |
|       the GrAxisChar structure and mask. Upon return,       |
|       the "axischar" structure will be filled in            |
|       with all current values of the graph's General        |
|       Characteristics dialog. Note that the "label_text"    |
|       member of "axischar" is dynamically allocated         |
|       and should be freed when no longer needed by the      |
|       client. Returns 1 if successful, 0 otherwise.         |
|                                                             |
*----------------Modification Audit/Change Log----------------*
| Rev  Sar                                                    |
| No   Code  Date      Rellvl  Mod  Description               |
| --   ----- --------  ------  ---  --------------------------|
|                                                             |
*-------------------------------------------------------------*/

/*
 * $Log: nexsAxisChar.c,v $
 * Revision 1.1  1998/03/23 02:58:36  tkm
 * Initial revision
 *
 */

#include "tclNExSInt.h"

static int axisDO( int argc, char **argv, int port, int gid, int axis);

int NExSConnObj_axis_char_cmd(
        void *clientData,
        Tcl_Interp *interp,
        int argc,
        char **argv)
{

NExSConnection * conn = (NExSConnection *) clientData;
int port=0;
int gid=0;
int axis=0;


	if (argc < 4 ) { 
		 Tcl_AppendResult( interp,
                "wrong # args: gid x|y|z values  ", argv[0],
                (char*) NULL);
                return (TCL_ERROR);
	}

	port=conn->port;
	Tcl_GetInt(interp, argv[1],&gid);
	axis=get_axis( argv[2] );

	return axisDO( argc-3, argv+3, port, gid, axis);
}




static int axisDO( 
	int argc, 
	char **argv, 
	int port, 
	int gid, 
	int axis) {

    unsigned long mask=0;
    GrAxisChar axischar;



	if (!strcmp(argv[0], "-axislabel"))
	    {
	    if (argc > 1)
		{
		mask = LABEL_TEXT;
		axischar.label_text = argv[1];

		}
	    }
	else if (!strcmp(argv[0], "-labelfont"))
	    {
	    if (argc < 1) return TCL_ERROR;
	    if (argc > 1)
		{
		mask = LABEL_FONT;
		axischar.label_font = atoi(argv[1]);
		}
	    }
	else if (!strcmp(argv[0], "-numticks"))
	    {
	    if (argc < 1) return TCL_ERROR;
	    if (argc > 1)
		{
		mask = LABEL_FONT;
		axischar.num_ticks = atoi(argv[1]);
		}
	    }
	else if (!strcmp(argv[0], "-logscale"))
	    {
	    if (argc < 1) return TCL_ERROR;
	    if (argc > 1)
		{
		mask = LOG_SCALE;
		axischar.log_scale = atoi(argv[1]);
		}
	    }
	else if (!strcmp(argv[0], "-autoticks"))
	    {
	    if (argc < 1) return TCL_ERROR;
	    mask = AUTO_TICKS;
	    axischar.auto_ticks = 1;
	    }
	else if (!strcmp(argv[0], "-setticks"))
	    {
	    if (argc < 1) return TCL_ERROR;
	    if (argc > 1)
		{
		mask = AUTO_TICKS | NUM_TICKS;
		axischar.auto_ticks = 0;
		axischar.num_ticks = atoi(argv[1]);
		}
	    }
	else if (!strcmp(argv[0], "-autoscale"))
	    {
	    if (argc < 1) return TCL_ERROR;
	    mask = AUTO_SCALE;
	    axischar.auto_scale = 1;
	    }
	else if (!strcmp(argv[0], "-setscale"))
	    {
	    if (argc < 1) return TCL_ERROR;
	    if (argc > 2)
		{
		mask = AUTO_SCALE | AXIS_MIN | AXIS_MAX;
		axischar.auto_scale = 0;
		axischar.axis_min = atof(argv[1]);
		axischar.axis_max = atof(argv[2]);
		}
	    }
	else if (!strcmp(argv[0], "-outticks"))
	    {
	    if (argc < 1) return TCL_ERROR;
	    if (argc > 1)
		{
		mask = OUT_TICKS;
		axischar.out_ticks = atoi(argv[1]);
		}
	    }
	else if (!strcmp(argv[0], "-exploded"))
	    {
	    if (argc < 1) return TCL_ERROR;
	    if (argc > 1)
		{
		mask = EXPLODED_AXIS;
		axischar.exploded_axis = atoi(argv[1]);
		}
	    }
	else if (!strcmp(argv[0], "-gridlines"))
	    {
	    if (argc < 1) return TCL_ERROR;
	    if (argc > 1)
		{
		mask = GRID_LINES;
		axischar.grid_lines = atoi(argv[1]);
		}
	    }


	    if (nexs_axis_char(port, gid, axis, &axischar, mask))
		{
			return TCL_OK;
		}
	return TCL_ERROR;
}


