static char copyright []
= "$Id: nexsDialog.c,v 1.1 1995/11/23 20:27:58 tkm Exp $\n\
   Copyright (c) 1992 General Electric.  All rights reserved.";

/*
 *   Permission to use, copy, modify, and distribute this
 *   software and its documentation for any purpose and without
 *   fee is hereby granted, provided that the above copyright
 *   notice appear in all copies and that both that copyright
 *   notice and this permission notice appear in supporting
 *   documentation, and that the name of General Electric not be used in
 *   advertising or publicity pertaining to distribution of the
 *   software without specific, written prior permission.
 *   General Electric makes no representations about the suitability of
 *   this software for any purpose.  It is provided "as is"
 *   without express or implied warranty.
 *
 *   This work was supported by the DARPA Initiative in Concurrent
 *   Engineering (DICE) through DARPA Contract MDA972-88-C-0047.
 *
 * $Log: nexsDialog.c,v $
 * Revision 1.1  1995/11/23 20:27:58  tkm
 * Initial revision
 *
 * Revision 1.1  1994/08/26  13:48:27  johnsonm
 * Initial revision
 *
 *
 * Old log:
 * Revision 1.2  1994/02/28  20:51:41  kennykb
 * Removed #includes that are now obtained from tclNExSInt.h
 *
 * Changed the Tk_ConfigureWidget call for the dialog box so that it
 * uses Tk_MainWindow to get the main window for the application, rather than
 * needing to store it in the NExSConnection structure.
 *
 * Revision 1.1  1992/10/09  19:04:27  kennykb
 * Initial revision
 *
 *
 * nexsDialog.c --
 *
 *	Functions to control NExS dialog boxes.
 */

#include "tclNExSInt.h"

/* Pointer to the connection object that is maintaining the current dialog */

static
NExSConnection * NExSCurrentDialog = (NExSConnection *) NULL;

/* Data structure controlling the current dialog */

static
struct NExSDialog {
  char * busycommand;		/* Tcl callback for BUSY status */
  char * cancelcommand;		/* Tcl callback for CANCEL status */
  char * command;		/* Tcl callback for OK status */
  char * dfault;		/* Default value for user input */
  char * prompt;		/* Message for prompt */
} NExSDialog;

/* Options for the `dialogue' command */

static Tk_ConfigSpec nexsDialogOpts [] = {
  {TK_CONFIG_STRING, "-busycommand", "nexsDialogBusyCommand",
     "NExSDialogBusyCommand", (char *) NULL,
     Tk_Offset (struct NExSDialog, busycommand), TK_CONFIG_NULL_OK},
  {TK_CONFIG_STRING, "-cancelcommand", "nexsDialogCancelCommand",
     "NExSDialogCancelCommand", (char *) NULL,
     Tk_Offset (struct NExSDialog, cancelcommand), TK_CONFIG_NULL_OK},
  {TK_CONFIG_STRING, "-command", "nexsDialogCommand", "NExSDialogCommand",
     (char *) NULL, Tk_Offset (struct NExSDialog, command), TK_CONFIG_NULL_OK},
  {TK_CONFIG_STRING, "-default", "nexsDialogDefault", "NExSDialogDefault",
     "", Tk_Offset (struct NExSDialog, dfault), 0},
  {TK_CONFIG_STRING, "-prompt", "nexsDialogPrompt", "NExSDialogPrompt",
     "Input requested from application.",
     Tk_Offset (struct NExSDialog, prompt), 0},
  {TK_CONFIG_END}
};

/* Static functions in this file */

static void NExSDialogCallback _ANSI_ARGS_((int, char *));
				/* Callback when a dialog completes */

/*
 * nexsConnObj_dialogue_cmd --
 *
 *	Display a dialogue box to prompt the user for the answer to a question.
 *
 * Syntax:
 *
 *	connectionName dialogue ?-option value?... 
 *
 * Where:
 *
 *	connectionName is the name of an NExS connection object
 *
 * Options:
 *
 *	-prompt TEXT
 *		Specifies the text with which to prompt the user.
 *		Default is `Input requested from application.', which
 *		isn't terribly informative
 *
 *	-default TEXT
 *		Specifies the data with which to pre-initialize the entry
 *		area of the dialog box.  Default is the empty string.
 *
 *	-busycommand COMMAND
 *		Specifies the Tcl command to execute if the dialog fails
 *		because NExS had another dialog in progress.  Default is
 *		the null string, which does nothing.
 *
 *	-cancelcommand COMMAND
 *		Specifies the Tcl command to execute if the dialog fails
 *		because the user pressed `Cancel'.  Default is the null
 *		string, which does nothing.
 *
 *	-command COMMAND
 *		Specifies the Tcl command to execute if the dialog succeeds.
 *
 * Notes:
 *	All the options allow for standard % substitutions.  The %m
 *	substitution returns the user's input.
 *
 * Result:
 *	Returns a standard Tcl result.
 *
 * Bugs:
 *	The connection library appears not to handle the `default' input
 *	properly.  It's probably a better idea to build the dialog box with
 *	Tk, anyway...
 */

int
NExSConnObj_dialogue_cmd (clientData, interp, argc, argv)
     ClientData clientData;
     Tcl_Interp * interp;
     int argc;
     char * * argv;
{
  NExSConnection * conn = (NExSConnection *) clientData;

  int status;
  Tk_Window win;

  /* Make sure that there isn't another dialog box up.  The connection
     library can't do multiple dialogs at a time. */

  if (NExSCurrentDialog != NULL) {
    Tcl_SetResult (interp, "another dialog is active", TCL_STATIC);
    return TCL_ERROR;
  }

  /* Parse the arguments */

  status = Tk_ConfigureWidget (interp, Tk_MainWindow (interp), nexsDialogOpts,
			       argc-1, argv+1, (char *) &NExSDialog, 0);
  if (status != TCL_OK)
    return status;

  /* Launch the dialog. */

  NExSCurrentDialog = conn;
#ifdef DEBUG_PRINTF
  fprintf (stderr, "\nnexs_dialog, port %d prompt \"%s\" default \"%s\"\n",
	   conn -> port, NExSDialog.prompt, NExSDialog.dfault);
#endif
  nexs_dialog (conn -> port, NExSDialog.prompt, NExSDialog.dfault,
	       NExSDialogCallback);

  /* All done until the connection library gets back to us. */

  return TCL_OK;
}

/*
 * NExSDialogCallback --
 *
 *	Handle the callback from the NExS connection library when an NExS
 * 	dialog completes.
 */

static void
NExSDialogCallback (status, string)
     int status;
     char * string;
{
  char * command;
  int tclStatus;

  /* Make sure that we know about the callback.  Otherwise, things
     could get weird. */

  if (NExSCurrentDialog == NULL) {
    fprintf (stderr,
	     "\nNExS dialog callback received when no dialog was active!?\n");
    abort ();
  }

  /* Decode the status */

  switch (status)
    {
    case NEXS_DIALOG_OK:
      command = NExSDialog.command;
      break;
    case NEXS_DIALOG_CANCELED:
      command = NExSDialog.cancelcommand;
      break;
    default:
      command = NExSDialog.busycommand;
      break;
    }

  /* Execute a callback into the Tcl environment */

  if (command != NULL) {
    if (string != NULL)
      NExSCallbackMessage = string;
    else
      NExSCallbackMessage = "";
    Tcl_ResetResult (NExSCurrentDialog -> interp);
    tclStatus = NExSEvalCallback (NExSCurrentDialog,
				  NExSCurrentDialog -> interp,

				  command, "CDNPWm");
    if (tclStatus != TCL_OK) {
      nexs_display_message (NExSCurrentDialog -> port,
			    NExSCurrentDialog -> interp -> result);
    }
    Tcl_ResetResult (NExSCurrentDialog -> interp);
    NExSCallbackMessage = "";
  }

  /* Release the dialog */

  NExSDialogCleanup (NExSCurrentDialog);
}

/*
 * NExSDialogCleanup --
 *
 *	Release storage occupied by an NExS dialog when it terminates, either
 *	with its callback or because its connection was closed.
 */

void
NExSDialogCleanup (conn)
     NExSConnection * conn;
{
  /* Make sure we're working with the right connection */

  if (conn != NExSCurrentDialog)
    return;

  /* Free storage */

  if (NExSDialog.busycommand != NULL)
    (void) ckfree (NExSDialog.busycommand);
  NExSDialog.busycommand = NULL;
  if (NExSDialog.cancelcommand != NULL)
    (void) ckfree (NExSDialog.cancelcommand);
  NExSDialog.cancelcommand = NULL;
  if (NExSDialog.command != NULL)
    (void) ckfree (NExSDialog.command);
  NExSDialog.command = NULL;
  if (NExSDialog.dfault != NULL)
    (void) ckfree (NExSDialog.dfault);
  NExSDialog.dfault = NULL;
  if (NExSDialog.prompt != NULL)
    (void) ckfree (NExSDialog.prompt);
  NExSDialog.prompt = NULL;

  /* Dialog no longer in progress */

  NExSCurrentDialog = (NExSConnection *) NULL;
}
