static char rcsid[] = "$Id: nexsGdataChar.c,v 1.1 1998/03/23 02:58:36 tkm Exp $";

/*------------------------------------------------------------*
|                                                             |
| Interface for NExS  gtaph API     .                         |
| Author:	Damir Delija                                  |
|               University Computing Center, SRCE             |
| Date:         Wed Mar 12 1997                               |
|               Copyright (c) 1997,  Damir Delija             |
|                                                             |
*--------------------Procedure Description--------------------*
|                                                             |
| nexs_gdata_graph.c D. Delija 12/03/97                       |
|                                                             |
| int nexs_gdata_char(int port, int gid, int dataset,         |
| NEXSGrDataChar *dchar, unsigned long mask);                 |
|                                                             |
|       This function provides access to all of the features  |
|       on the Graph Data Set Characteristics dialog. It      |
|       is similar to the NExS v2.1 compatible call           |
|       !xs2_gdata_char(), except that it provides access to   |
|       all of the version 3 data set features, and it DOES   |
|       NOT use dynmically allocated strings. DO NOT free     |
|       strings returned in the NEXSGrDataChar structure.     |
|       The structure members and mask values are defined in  |
|       "nexs.h".  Example:                                   |
|                                                             |
|       Set the legend text of dataset 3 on graph 1           |
|       to 18pt Helvetica bold italic                         |
|                                                             |
|         NEXSGrDataChar dchar;                               |
|         unsigned long mask;                                 |
|         dchar.legend_size = NEXS_FONT_SIZE_18PT;            |
|         dchar.legend_family = NEXS_FONT_FAMILY_HELVETICA;   |
|         dchar.legend_style = NEXS_FONT_STYLE_BOLDITALIC;    |
|         mask = NEXS_GDATA_LEGEND_SIZE |                     |
|                NEXS_GDATA_LEGEND_FAMILY |                   |
|                NEXS_GDATA_LEGEND_STYLE;                     |
|         nexs_gdata_char(port, 1, 3, &dchar, mask);          |
|                                                             |
*----------------Modification Audit/Change Log----------------*
| Rev  Sar                                                    |
| No   Code  Date      Rellvl  Mod  Description               |
| --   ----- --------  ------  ---  --------------------------|
|                                                             |
*-------------------------------------------------------------*/

/*
 * $Log: nexsGdataChar.c,v $
 * Revision 1.1  1998/03/23 02:58:36  tkm
 * Initial revision
 *
 */


#include "tclNExSInt.h"

static int gdataDO ( int argc, char **argv, int port, int gid, int dataset);

int NExSConnObj_gdata_char_cmd (
        void *clientData,
        Tcl_Interp *interp,
        int argc,
        char **argv)
{

NExSConnection * conn = (NExSConnection *) clientData;
int port=0;
int gid=0;
int dataset=0;

	if (argc < 3 ) { 
		 Tcl_AppendResult( interp,
                "wrong # args: gid dataset values  ", argv[0],
                (char*) NULL);
                return (TCL_ERROR);
	}
	port=conn->port;
	Tcl_GetInt(interp, argv[1],&gid);
	Tcl_GetInt(interp, argv[2],&dataset);

	return gdataDO(argc-3,argv+3,port,gid,dataset);
}


static int gdataDO (
	int argc, 
	char **argv, 
	int port, 
	int gid,
	int dataset) {

    Range src, dst;
    unsigned long mask=0;
    GrGenChar genchar;
    GrAxisChar axischar;
    XS2GrDataChar dchar;
    GrRotate3D grotate;
    GrGeometry ggeom;
    int axis;  

	if (!strcmp(argv[0], "-datarange"))
	    {
	    if (argc < 2) return TCL_ERROR;
	    axis = get_axis(argv[1]);
	    if (argc > 2)
		{
		switch (axis)
		    {
		    case 0:	mask = X_RANGE;
				dchar.x_range = argv[2];
				break;
		    case 1:	mask = Y_RANGE;
				dchar.y_range = argv[2];
				break;
		    case 2:	mask = Z_RANGE;
				dchar.z_range = argv[2];
				break;
		    }
		}
	    }
	else if (!strcmp(argv[0], "-legendtext"))
	    {
	    if (argc > 1)
		{
		dchar.legend_text = argv[1];
		mask = LEGEND_TEXT;
		}
	    }
	else if (!strcmp(argv[0], "-markertype"))
	    {
	    if (argc > 1)
		{
		dchar.marker_type = atoi(argv[1]);
		mask = MARKER_TYPE;
		}
	    }
	else if (!strcmp(argv[0], "-linestyle"))
	    {
	    if (argc > 1)
		{
		dchar.line_style = atoi(argv[1]);
		mask = LINE_STYLE;
		}
	    }
	else if (!strcmp(argv[0], "-segmenttype"))
	    {
	    if (argc > 1)
		{
		dchar.segment_type = atoi(argv[1]);
		mask = SEGMENT_TYPE;
		}
	    }
	else if (!strcmp(argv[0], "-disable"))
	    {
	    if (argc > 1)
		{
		dchar.data_disable = atoi(argv[1]);
		mask = DATA_DISABLE;
		}
	    }

	if (!xs2_gdata_char(port, gid, dataset, &dchar, mask))
		{
			return TCL_OK;
		}
	return TCL_ERROR;
}
