static char copyright []
= "$Id: nexsGetPrinttype.c,v 1.1 1995/11/23 20:27:58 tkm Exp $\n\
   Copyright (c) 1992 General Electric.  All rights reserved.";

/*
 *   Permission to use, copy, modify, and distribute this
 *   software and its documentation for any purpose and without
 *   fee is hereby granted, provided that the above copyright
 *   notice appear in all copies and that both that copyright
 *   notice and this permission notice appear in supporting
 *   documentation, and that the name of General Electric not be used in
 *   advertising or publicity pertaining to distribution of the
 *   software without specific, written prior permission.
 *   General Electric makes no representations about the suitability of
 *   this software for any purpose.  It is provided "as is"
 *   without express or implied warranty.
 *
 *   This work was supported by the DARPA Initiative in Concurrent
 *   Engineering (DICE) through DARPA Contract MDA972-88-C-0047.
 *
 * $Log: nexsGetPrinttype.c,v $
 * Revision 1.1  1995/11/23 20:27:58  tkm
 * Initial revision
 *
 * Revision 1.1  1994/08/26  14:03:59  johnsonm
 * Initial revision
 *
 *
 * Old log:
 * Revision 1.2  1994/02/28  20:57:27  kennykb
 * Removed #includes that are now obtained from tclNExSInt.h
 *
 * Revision 1.1  1992/10/09  19:07:48  kennykb
 * Initial revision
 *
 *
 * nexsGetPrinttype.c --
 *
 *	This file contains all the support needed to parse NExS printtype
 *	specifications.
 */

#include "tclNExSInt.h"

static char * nexsParsePrinttype _ANSI_ARGS_((char *, int *));
				/* Service function that does the work */

static char * parseError;	/* Error message from the parse */

/*
 * nexsGetPrinttype --
 *	Get an NExS printtype code and return it in `printtype'.
 *	Returns a Tcl status.
 */

int
nexsGetPrinttype (interp, string, printtype)
     Tcl_Interp * interp;
     char * string;
     int * printtype;
{
  char * result;
  result = nexsParsePrinttype (string, printtype);
  if (result == NULL)
    return TCL_OK;
  Tcl_SetResult (interp, result, TCL_STATIC);
  return TCL_ERROR;
}

/*
 * nexsGetPrinttypeArgv --
 *
 * 	Do nexsGetPrinttype from the context of Tk_ParseArgv.
 */

TK_MAKE_GENFUNC( nexsGetPrinttypeArgv, int, nexsGetPrinttype )

/*
 * nexsParsePrinttype --
 *
 *	This function does all the real work of parsing a print type
 */

static char *
nexsParsePrinttype (string, printtype)
     char * string;
     int * printtype;
{
  int len;
  len = strlen (string);
  if (strncmp (string, "ascii", len) == 0) {
    *printtype = PRINT_ASCII;
    return NULL;
  }
  else if (strncmp (string, "ps", len) == 0) {
    *printtype = PRINT_PS;
    return NULL;
  }
  else if (strncmp (string, "postscript", len) == 0) {
    *printtype = PRINT_PS;
    return NULL;
  }
  else {
    return "unknown print type -- must be \"ascii\" or \"ps\"";
  }
}
