/*
 * $Id: tclNExSInt.h,v 1.1 1995/11/23 20:27:58 tkm Exp $
 *
 * Copyright (c) 1992 General Electric.  All rights reserved.
 *
 *   Permission to use, copy, modify, and distribute this
 *   software and its documentation for any purpose and without
 *   fee is hereby granted, provided that the above copyright
 *   notice appear in all copies and that both that copyright
 *   notice and this permission notice appear in supporting
 *   documentation, and that the name of General Electric not be used in
 *   advertising or publicity pertaining to distribution of the
 *   software without specific, written prior permission.
 *   General Electric makes no representations about the suitability of
 *   this software for any purpose.  It is provided "as is"
 *   without express or implied warranty.
 *
 *   This work was supported by the DARPA Initiative in Concurrent
 *   Engineering (DICE) through DARPA Contract MDA972-88-C-0047.
 *
 * $Log: tclNExSInt.h,v $
 * Revision 1.1  1995/11/23 20:27:58  tkm
 * Initial revision
 *
 * Revision 1.2  1994/11/14  20:49:36  johnsonm
 * Commented #endif comment.
 *
 * Revision 1.1  1994/08/26  14:35:02  johnsonm
 * Initial revision
 *
 *
 * Old log:
 * Revision 1.4  1994/03/02  22:52:05  kennykb
 * Changed nexsGetRow and nexsGetColumn to used unsigned row and column
 * numbers.  The NExS connection library is inconsistent.
 *
 * Revision 1.3  1994/03/02  21:19:16  kennykb
 * Portability changes for autoconf.
 *
 * Revision 1.2  1994/02/28  20:34:30  kennykb
 * Added <stdlib.h> and <string.h> to the standard set of included files.
 * FIXME: This will have to be revised in autoconfig.
 *
 * Removed display and application main window from the NExSConnection
 * structure, but added a Tk_UID that identifies the screen.  Display is now
 * managed in the Tk library by opening a dummy window on the display,
 * retrieving the display pointer from it, and closing the dummy window again.
 * This kludge has the side effect of establishing Tk event management ot
 * the display.
 *
 * Removed the NExSDisplayOption parser and printer.  Display is parsed
 * now only by Tk_CreateWindow.
 *
 * Added a conditional declaration for `free', to avoid errors if stdlib.h
 * is broken (with `free' missing), and the only reference to `free' is in
 * the TCL_DYNAMIC macro expansion.
 *
 * Revision 1.1  1992/10/09  18:55:59  kennykb
 * Initial revision
 *
 *
 * tclNExSInt.h --
 *
 *	This file contains definitions of all the internals needed in
 * the interface between Tcl and the NEXS API.
 *
 */

#if STDC_HEADERS || HAVE_STDLIB_H
#include <stdlib.h>
#endif

/* We need to use somebody's string library. */

#if STDC_HEADERS || HAVE_STRING_H
#include <string.h>
/* An ANSI string.h and a pre-ANSI memory.h might conflict */
#if !STDC_HEADERS && HAVE_MEMORY_H
#include <memory.h>
#endif
#else
#include <strings.h>
#define strchr index
#define strrchr rindex
#endif

#include <tcl.h>
#include <tk.h>
#include "genfunc.h"
#include "nexs.h"
#ifndef NeedFunctionPrototypes
#include "nexsfns.h"
#endif

/* Configuration parameters */

#define NEXS_PORTS 64		/* Maximum number of NExS connection ports */

/* Data structure describing an NExS connection */

typedef struct NExSConnection {
  Tcl_Interp * interp;		/* Interpreter in which connection was
				 * opened */
  char * objectName;		/* Name of the connection object command */
  Tk_Uid screenUid;		/* Name of the screen on which NExS is
				 * running */
  int dispID;			/* NExS internal identifier for the display */
  char * name;			/* Connection name */
  int port;			/* NExS port ID for the connection */
  int timeout;			/* Timeout for connection */
  Window window;		/* X window ID for the instance of NExS */
  char * closeCmd;		/* Close command */
  char * messageCmd;		/* Message command */
  struct NExSMenu * firstMenu;	/* First menu in the spreadsheet */
  struct NExSFunction * firstFunction;
				/* First external function installed in the */
				/* sheet. */
  int flags;			/* Set of flags for connection state */
} NExSConnection;

/* Bits in the `flags' word above */

#define NEXS_CONNECTION_CLOSED	(1<<0)
				/* Flag: connection closed at other end */
#define NEXS_FUNCTION_PENDING	(1<<1)
				/* Flag: external function evaluation in */
				/* progress. */
#define NEXS_CONNECTION_LOCKED	(1<<2)
				/* Flag: nexs_user_lock has been set */

/* Table of active connections indexed by port */

extern NExSConnection * NExSConnTable [NEXS_PORTS];

/* Data structure describing an NExS menu */

typedef struct NExSMenu {
  NExSConnection * conn;	/* Connection to which this menu belongs */
  long menuID;			/* NEXS menu ID of this menu */
  char * objectName;		/* Name of the menu object command */
  char * name;			/* Name of the menu in menubar */
  struct NExSButton * firstButton;
				/* First button in the menu */
  struct NExSMenu * next, * prev;
				/* Next and previous menu in the spreadsheet */
} NExSMenu;

/* Data structure describing an NExS menubutton */

typedef struct NExSButton {
  NExSMenu * menu;		/* Menu to which this button pertains */
  long buttonID;		/* NEXS button ID of this button */
  char * objectName;		/* Name of the button object command */
  char * name;			/* Name of the button within the menu */
  char * command;		/* Command to execute when button pressed */
  struct NExSButton * next, * prev;
				/* Next and previous button in the menu */
} NExSButton;

/* Data structure describing an NExS external function */

typedef struct NExSFunction {
  NExSConnection * conn;	/* Connection to which this function belongs */
  char * name;			/* Name of the function within NExS */
  char * function;		/* Tcl function to execute when function is */
				/* invoked */
  struct NExSFunction * next, * prev;
				/* Next and previous function */
} NExSFunction;

/* Data structure describing an NExS cell */

typedef struct NExSCell {
  int row;			/* Row number */
  int col;			/* Column number */
  int flags;			/* Flags */
} NExSCell;

#define NEXS_CELL_ABSROW (1<<0)	/* Absolute row address */
#define NEXS_CELL_ABSCOL (1<<1)	/* Absolute column address */


#ifdef notdef
/* Tk option parser and printer for the `-display' option on commands */

extern TK_DECLARE_CUSTOM_PARSE( NExSParseDisplay )
extern char * NExSPrintDisplay _ANSI_ARGS_((ClientData, Tk_Window,
					    char *, int, Tcl_FreeProc **));

extern Tk_CustomOption NExSDisplayOption;
#endif

/* The parser for NExS cell addresses */

extern int nexsGetRange _ANSI_ARGS_((Tcl_Interp * interp, char * address,
				     Range * range));
extern TK_DECLARE_GENFUNC( nexsGetRangeArgv )
extern int nexsGetCell _ANSI_ARGS_((Tcl_Interp *, char *, NExSCell *));
extern TK_DECLARE_GENFUNC( nexsGetCellArgv )
extern int nexsGetColumn _ANSI_ARGS_((Tcl_Interp * interp, char * colname,
				      int flags, unsigned * col,
				      int * retflags,
				      char ** next));
extern TK_DECLARE_GENFUNC( nexsGetColumnArgv )
extern int nexsGetRow _ANSI_ARGS_((Tcl_Interp * inter, char * rowname,
				   int flags, unsigned * row, int * retflags,
				   char ** next));
extern TK_DECLARE_GENFUNC( nexsGetRowArgv )

/* Re-encode cell addresses */

extern int nexsStoreRange _ANSI_ARGS_((Tcl_Interp * interp, Range * range,
				       char * rangestring, char * * next));
extern int nexsStoreCell _ANSI_ARGS_((Tcl_Interp * interp, int row, int col,
				      int flags,
				      char * cellname, char * * next));
extern int nexsStoreColumn _ANSI_ARGS_((Tcl_Interp * interp, int col,
					int flags,
					char * colname, char * * next));
extern int nexsStoreRow _ANSI_ARGS_((Tcl_Interp * interp, int row, int flags,
				     char * rowname, char * * next));

/* The parser for NExS font IDs */

extern int nexsGetFont _ANSI_ARGS_((Tcl_Interp *, char *, int *));
extern TK_DECLARE_GENFUNC( nexsGetFontArgv )

/* The parser for NExS format codes */

extern int nexsGetFormat _ANSI_ARGS_((Tcl_Interp *, char *, int *));
extern TK_DECLARE_GENFUNC( nexsGetFormatArgv )

/* The parser for NExS print types */

extern int nexsGetPrinttype _ANSI_ARGS_((Tcl_Interp *, char *, int *));
extern TK_DECLARE_GENFUNC( nexsGetPrinttypeArgv )

/* The parser for NExS file types */

extern int nexsGetFiletype _ANSI_ARGS_((Tcl_Interp *, char *, int *));
extern TK_DECLARE_GENFUNC( nexsGetFiletypeArgv )

/* Tk_ConfigureWidget custom option procedures for flagging whether an
 * option is present. */

extern TK_DECLARE_CUSTOM_PARSE( nexsOptionFlagParseProc )
extern int nexsOptionFlagParseProc _ANSI_ARGS_((ClientData, Tcl_Interp *,
						Tk_Window, char *, char *,
						int));
extern char * nexsOptionFlagPrintProc _ANSI_ARGS_((ClientData, Tk_Window,
						   char *, int,
						   Tcl_FreeProc * *));

#define NEXS_OPTION_BIT(name,value)					\
static Tk_CustomOption name = {						\
  nexsOptionFlagParseProc,						\
  nexsOptionFlagPrintProc,						\
  (ClientData) value							\
}

#define NEXS_OPTION_FLAG(widget,word,bit)				\
  { TK_CONFIG_CUSTOM, (char *) NULL, (char *) NULL, (char *) NULL,	\
    (char *) NULL, Tk_Offset(widget,word), 0, &bit }

/* Callback for NExS event processing */

extern Bool NExSEventFilter _ANSI_ARGS_((ClientData, XEvent *));

/* NExS callback entries */

extern void NExSCloseCallback _ANSI_ARGS_((int port));
extern void NExSReceiveCallback _ANSI_ARGS_((int port, char message[15]));

/* Procedure to execute a Tcl callback from NExS */

extern int NExSEvalCallback _ANSI_ARGS_((NExSConnection *,
					 Tcl_Interp *,
					 char *,
					 char *));
extern NExSButton * NExSCallbackButton;
				/* Button that caused a callback to be
				   invoked */
extern char * NExSCallbackMessage;
				/* Interclient message to be reported */

/* Tcl object commands */

extern int NExSConnObj_cmd _ANSI_ARGS_((ClientData, Tcl_Interp *,
					int, char * *));

/* External cleanup entries needed for shutting things down gracefully */

extern void NExSConnObj_Destroy _ANSI_ARGS_((ClientData));
				/* Destroy a connection object */
extern void NExSFunctObj_Delete _ANSI_ARGS_((struct NExSFunction *));
				/* Destroy an external function binding */
extern void NExSMenuObj_Delete _ANSI_ARGS_((struct NExSMenu *));
				/* Destroy a menu object. */
extern void NExSDialogCleanup _ANSI_ARGS_((NExSConnection *));
				/* Clean up dialog box associated with a */
				/* connection */

#ifdef FREEBUG
/* TCL_DYNAMIC doesn't function unless a declaration of `free' is present
 * somewhere...
 */

extern void free _ANSI_ARGS_((char *));
#endif /* FREEBUG */

